from enum import Enum, unique


@unique
class Industry(Enum):
    ADVANCED_MEDICAL_EQUIPMENT_AND_TECHNOLOGY = "AdvancedMedicalEquipmentAndTechnology"
    ADVERTISING_AND_MARKETING = "AdvertisingAndMarketing"
    AEROSPACE_AND_DEFENSE = "AerospaceAndDefense"
    AGRICULTURAL_CHEMICALS = "AgriculturalChemicals"
    AIRLINES = "Airlines"
    AIRPORT_OPERATORS_AND_SERVICES = "AirportOperatorsAndServices"
    ALUMINUM = "Aluminum"
    APPAREL_AND_ACCESSORIES = "ApparelAndAccessories"
    APPAREL_AND_ACCESSORIES_RETAILERS = "ApparelAndAccessoriesRetailers"
    APPLIANCES_TOOLS_AND_HOUSEWARES = "AppliancesToolsAndHousewares"
    AUTO_AND_TRUCK_MANUFACTURERS = "AutoAndTruckManufacturers"
    AUTO_TRUCK_AND_MOTORCYCLE_PARTS = "AutoTruckAndMotorcycleParts"
    AUTO_VEHICLES_PARTS_AND_SERVICE_RETAILERS = "AutoVehiclesPartsAndServiceRetailers"
    BANKS = "Banks"
    BIOTECHNOLOGY_AND_MEDICAL_RESEARCH = "BiotechnologyAndMedicalResearch"
    BLOCKCHAIN_AND_CRYPTOCURRENCY = "BlockchainAndCryptocurrency"
    BREWERS = "Brewers"
    BROADCASTING = "Broadcasting"
    BUSINESS_SUPPORT_SERVICES = "BusinessSupportServices"
    BUSINESS_SUPPORT_SUPPLIES = "BusinessSupportSupplies"
    CASINOS_AND_GAMING = "CasinosAndGaming"
    CHARITY_ORGANIZATIONS = "CharityOrganizations"
    CIVIC_AND_SOCIAL_ORGANIZATIONS = "CivicAndSocialOrganizations"
    CLOSED_END_FUNDS = "ClosedEndFunds"
    COAL = "Coal"
    COMMERCIAL_PRINTING_SERVICES = "CommercialPrintingServices"
    COMMERCIAL_REI_TS = "CommercialREITs"
    COMMODITY_CHEMICALS = "CommodityChemicals"
    COMMUNICATIONS_AND_NETWORKING = "CommunicationsAndNetworking"
    COMPUTER_AND_ELECTRONICS_RETAILERS = "ComputerAndElectronicsRetailers"
    COMPUTER_HARDWARE = "ComputerHardware"
    CONSTRUCTION_AND_ENGINEERING = "ConstructionAndEngineering"
    CONSTRUCTION_MATERIALS = "ConstructionMaterials"
    CONSTRUCTION_SUPPLIES_AND_FIXTURES = "ConstructionSuppliesAndFixtures"
    CONSUMER_GOODS_CONGLOMERATES = "ConsumerGoodsConglomerates"
    CONSUMER_LENDING = "ConsumerLending"
    CONSUMER_PUBLISHING = "ConsumerPublishing"
    CORPORATE_FINANCIAL_SERVICES = "CorporateFinancialServices"
    COURIER_POSTAL_AIR_FREIGHT_AND_LANDBASED_LOGISTICS = (
        "CourierPostalAirFreightAndLandbasedLogistics"
    )
    CROWD_COLLABORATION = "CrowdCollaboration"
    DEPARTMENT_STORES = "DepartmentStores"
    DISCOUNT_STORES = "DiscountStores"
    DISTILLERS_AND_WINERIES = "DistillersAndWineries"
    DIVERSIFIED_CHEMICALS = "DiversifiedChemicals"
    DIVERSIFIED_INDUSTRIAL_GOODS_WHOLESALE = "DiversifiedIndustrialGoodsWholesale"
    DIVERSIFIED_INVESTMENT_SERVICES = "DiversifiedInvestmentServices"
    DIVERSIFIED_MINING = "DiversifiedMining"
    DIVERSIFIED_REI_TS = "DiversifiedREITs"
    DRUG_RETAILERS = "DrugRetailers"
    ELECTRIC_UTILITIES = "ElectricUtilities"
    ELECTRICAL_COMPONENTS_AND_EQUIPMENT = "ElectricalComponentsAndEquipment"
    ELECTRONIC_EQUIPMENT_AND_PARTS = "ElectronicEquipmentAndParts"
    EMPLOYMENT_SERVICES = "EmploymentServices"
    ENTERTAINMENT_PRODUCTION = "EntertainmentProduction"
    ENVIRONMENTAL_ORGANIZATIONS = "EnvironmentalOrganizations"
    ENVIRONMENTAL_SERVICES_AND_EQUIPMENT = "EnvironmentalServicesAndEquipment"
    EXCHANGE_TRADED_FUNDS = "ExchangeTradedFunds"
    FINANCIAL_AND_COMMODITY_MARKET_OPERATORS_AND_SERVICE_PROVIDERS = (
        "FinancialAndCommodityMarketOperatorsAndServiceProviders"
    )
    FINANCIAL_TECHNOLOGY = "FinancialTechnology"
    FISHING_AND_FARMING = "FishingAndFarming"
    FOOD_PROCESSING = "FoodProcessing"
    FOOD_RETAIL_AND_DISTRIBUTION = "FoodRetailAndDistribution"
    FOOTWEAR = "Footwear"
    FOREST_AND_WOOD_PRODUCTS = "ForestAndWoodProducts"
    GOLD = "Gold"
    GOVERNMENT_ADMINISTRATION_ACTIVITIES = "GovernmentAdministrationActivities"
    GOVERNMENT_AND_GOVERNMENT_FINANCE = "GovernmentAndGovernmentFinance"
    GROUND_FREIGHT_AND_LOGISTICS = "GroundFreightAndLogistics"
    HEALTHCARE_FACILITIES_AND_SERVICES = "HealthcareFacilitiesAndServices"
    HEAVY_ELECTRICAL_EQUIPMENT = "HeavyElectricalEquipment"
    HEAVY_MACHINERY_AND_VEHICLES = "HeavyMachineryAndVehicles"
    HIGHWAYS_AND_RAIL_TRACKS = "HighwaysAndRailTracks"
    HOME_FURNISHINGS = "HomeFurnishings"
    HOME_FURNISHINGS_RETAILERS = "HomeFurnishingsRetailers"
    HOME_IMPROVEMENT_PRODUCTS_AND_SERVICES_RETAILERS = (
        "HomeImprovementProductsAndServicesRetailers"
    )
    HOMEBUILDING = "Homebuilding"
    HOTELS_MOTELS_AND_CRUISE_LINES = "HotelsMotelsAndCruiseLines"
    HOUSEHOLD_ELECTRONICS = "HouseholdElectronics"
    HOUSEHOLD_PRODUCTS = "HouseholdProducts"
    IT_SERVICES_AND_CONSULTING = "ITServicesAndConsulting"
    INDEPENDENT_POWER_PRODUCERS = "IndependentPowerProducers"
    INDUSTRIAL_MACHINERY_AND_EQUIPMENT = "IndustrialMachineryAndEquipment"
    INSURANCE_FUNDS = "InsuranceFunds"
    INTEGRATED_HARDWARE_AND_SOFTWARE = "IntegratedHardwareAndSoftware"
    INTEGRATED_OIL_AND_GAS = "IntegratedOilAndGas"
    INTEGRATED_TELECOMMUNICATIONS_SERVICES = "IntegratedTelecommunicationsServices"
    INVESTMENT_BANKING_AND_BROKERAGE_SERVICES = "InvestmentBankingAndBrokerageServices"
    INVESTMENT_HOLDING_COMPANIES = "InvestmentHoldingCompanies"
    INVESTMENT_MANAGEMENT_AND_FUND_OPERATORS = "InvestmentManagementAndFundOperators"
    IRON_AND_STEEL = "IronAndSteel"
    LEGAL_AND_SAFETY_PUBLIC_SERVICES = "LegalAndSafetyPublicServices"
    LEISURE_AND_RECREATION = "LeisureAndRecreation"
    LIFE_AND_HEALTH_INSURANCE = "LifeAndHealthInsurance"
    MANAGED_HEALTHCARE = "ManagedHealthcare"
    MARINE_FREIGHT_AND_LOGISTICS = "MarineFreightAndLogistics"
    MARINE_PORT_SERVICES = "MarinePortServices"
    MEDICAL_EQUIPMENT_SUPPLIES_AND_DISTRIBUTION = (
        "MedicalEquipmentSuppliesAndDistribution"
    )
    MINING_SUPPORT_SERVICES_AND_EQUIPMENT = "MiningSupportServicesAndEquipment"
    MISCELLANEOUS_EDUCATIONAL_SERVICE_PROVIDERS = (
        "MiscellaneousEducationalServiceProviders"
    )
    MISCELLANEOUS_INFRASTRUCTURE = "MiscellaneousInfrastructure"
    MISCELLANEOUS_SPECIALTY_RETAILERS = "MiscellaneousSpecialtyRetailers"
    MULTILINE_INSURANCE_AND_BROKERS = "MultilineInsuranceAndBrokers"
    MULTILINE_UTILITIES = "MultilineUtilities"
    MUTUAL_FUNDS = "MutualFunds"
    NATIONAL_SECURITY_AND_INTERNATIONAL_AFFAIRS = (
        "NationalSecurityAndInternationalAffairs"
    )
    NATURAL_GAS_UTILITIES = "NaturalGasUtilities"
    NON_ALCOHOLIC_BEVERAGES = "NonAlcoholicBeverages"
    NON_GOLD_PRECIOUS_METALS_AND_MINERALS = "NonGoldPreciousMetalsAndMinerals"
    NON_PAPER_CONTAINERS_AND_PACKAGING = "NonPaperContainersAndPackaging"
    OFFICE_EQUIPMENT = "OfficeEquipment"
    OIL_AND_GAS_DRILLING = "OilAndGasDrilling"
    OIL_AND_GAS_EXPLORATION_AND_PRODUCTION = "OilAndGasExplorationAndProduction"
    OIL_AND_GAS_REFINING_AND_MARKETING = "OilAndGasRefiningAndMarketing"
    OIL_AND_GAS_TRANSPORTATION_SERVICES = "OilAndGasTransportationServices"
    OIL_RELATED_SERVICES_AND_EQUIPMENT = "OilRelatedServicesAndEquipment"
    ONLINE_SERVICES = "OnlineServices"
    PAPER_PACKAGING = "PaperPackaging"
    PAPER_PRODUCTS = "PaperProducts"
    PASSENGER_TRANSPORTATION_GROUND_AND_SEA = "PassengerTransportationGroundAndSea"
    PENSION_FUNDS = "PensionFunds"
    PERSONAL_PRODUCTS = "PersonalProducts"
    PERSONAL_SERVICES = "PersonalServices"
    PHARMACEUTICALS = "Pharmaceuticals"
    PHONES_AND_HANDHELD_DEVICES = "PhonesAndHandheldDevices"
    PROFESSIONAL_AND_BUSINESS_EDUCATION = "ProfessionalAndBusinessEducation"
    PROFESSIONAL_INFORMATION_SERVICES = "ProfessionalInformationServices"
    PROFESSIONAL_ORGANIZATIONS = "ProfessionalOrganizations"
    PROPERTY_AND_CASUALTY_INSURANCE = "PropertyAndCasualtyInsurance"
    REAL_ESTATE_RENTAL_DEVELOPMENT_AND_OPERATIONS = (
        "RealEstateRentalDevelopmentAndOperations"
    )
    REAL_ESTATE_SERVICES = "RealEstateServices"
    RECREATIONAL_PRODUCTS = "RecreationalProducts"
    REINSURANCE = "Reinsurance"
    RELIGIOUS_ORGANIZATIONS = "ReligiousOrganizations"
    RENEWABLE_ENERGY_EQUIPMENT_AND_SERVICES = "RenewableEnergyEquipmentAndServices"
    RENEWABLE_FUELS = "RenewableFuels"
    RESIDENTIAL_REI_TS = "ResidentialREITs"
    RESTAURANTS_AND_BARS = "RestaurantsAndBars"
    SCHOOL_COLLEGE_AND_UNIVERSITY = "SchoolCollegeAndUniversity"
    SEMICONDUCTOR_EQUIPMENT_AND_TESTING = "SemiconductorEquipmentAndTesting"
    SEMICONDUCTORS = "Semiconductors"
    SHIPBUILDING = "Shipbuilding"
    SOFTWARE = "Software"
    SPECIALIZED_REI_TS = "SpecializedREITs"
    SPECIALTY_CHEMICALS = "SpecialtyChemicals"
    SPECIALTY_MINING_AND_METALS = "SpecialtyMiningAndMetals"
    TEXTILES_AND_LEATHER_GOODS = "TextilesAndLeatherGoods"
    TIRES_AND_RUBBER_PRODUCTS = "TiresAndRubberProducts"
    TOBACCO = "Tobacco"
    TOYS_AND_CHILDREN_PRODUCTS = "ToysAndChildrenProducts"
    UK_INVESTMENT_TRUSTS = "UKInvestmentTrusts"
    URANIUM = "Uranium"
    WATER_AND_RELATED_UTILITIES = "WaterAndRelatedUtilities"
    WIRELESS_TELECOMMUNICATIONS_SERVICES = "WirelessTelecommunicationsServices"



from typing import TYPE_CHECKING

from .._content_data import Data
from .._content_provider_layer import ContentUsageLoggerMixin
from ..._content_type import ContentType
from ..._tools import hp_universe_parser, validate_types, try_copy_to_list
from ...delivery._data._data_provider import BaseResponse, DataProviderLayer

if TYPE_CHECKING:
    from ._enums import OptAdjustments, OptEventTypes
    from ..._types import OptDateTime, StrStrings, OptInt, ExtendedParams, OptStrStrs


class Definition(
    ContentUsageLoggerMixin[BaseResponse[Data]], DataProviderLayer[BaseResponse[Data]]
):
    """
    Summary line of this class that defines parameters for requesting events from historical pricing

    Parameters
    ----------
    universe : str or list of str
        The entity universe
    eventTypes : list of EventTypes or EventTypes or str, optional
        The market events EventTypes
    start : str or date or datetime or timedelta, optional
        The start date and timestamp of the query in ISO8601 with UTC only
    end : str or date or datetime or timedelta, optional
        The end date and timestamp of the query in ISO8601 with UTC only
    adjustments : list of Adjustments or Adjustments or str, optional
        The adjustment list or Adjustments type
    count : int, optional
        The maximum number of data returned. Values range: 1 - 10000
    fields : list, optional
        The list of fields that are to be returned in the response
    extended_params : dict, optional
        If necessary other parameters

    Examples
    --------
    >>> from refinitiv.data.content.historical_pricing import events
    >>> definition_events = events.Definition("EUR")
    >>> response = definition_events.get_data()

    """

    _USAGE_CLS_NAME = "HistoricalPricing.EventsDefinition"

    def __init__(
        self,
        universe: "StrStrings",
        eventTypes: "OptEventTypes" = None,
        start: "OptDateTime" = None,
        end: "OptDateTime" = None,
        adjustments: "OptAdjustments" = None,
        count: "OptInt" = None,
        fields: "OptStrStrs" = None,
        extended_params: "ExtendedParams" = None,
    ):
        validate_types(count, [int, type(None)], "count")
        universe = try_copy_to_list(universe)
        universe = hp_universe_parser.get_list(universe)
        event_types = try_copy_to_list(eventTypes)
        adjustments = try_copy_to_list(adjustments)
        fields = try_copy_to_list(fields)

        super().__init__(
            data_type=ContentType.HISTORICAL_PRICING_EVENTS,
            universe=universe,
            event_types=event_types,
            start=start,
            end=end,
            adjustments=adjustments,
            count=count,
            fields=fields,
            extended_params=extended_params,
        )



from typing import TYPE_CHECKING

from ._db_manager import (
    create_db_manager_by_package_name,
    DBManager,
)
from ...._tools import try_copy_to_list
from ...._tools._common import fields_arg_parser, universe_arg_parser

if TYPE_CHECKING:
    from ...._types import StrStrings, OptStrStrs


class Response:
    pass


class Definition:
    """
    This class describe parameters to retrieve ESG data.

    Parameters
    ----------
    package_name : str

    universe : str, list of str
        The Universe parameter allows the user to define the company they
        want content returned for, ESG content is delivered at the Company Level.

    fields : list, optional
        The list of fields that are to be returned in the response

    Examples
    --------
    >>> import refinitiv.data.content.esg.bulk as bulk
    >>> definition = bulk.Definition(
    ...     package_name='standard_scores',
    ...     universe=['4295875817', '4295889298'],
    ...     fields=["instrument", "periodenddate"]
    ... )
    >>> response = definition.get_db_data()
    >>> df = response.data.df
    >>> print(df)

    """

    def __init__(
        self,
        package_name: str,
        universe: "StrStrings",
        fields: "OptStrStrs" = None,
    ) -> None:
        fields = try_copy_to_list(fields)
        self._fields = fields and fields_arg_parser.get_list(fields)
        universe = try_copy_to_list(universe)
        self._universe = universe_arg_parser.get_list(universe)
        self._db_manager: DBManager = create_db_manager_by_package_name(package_name)

    def get_db_data(self) -> Response:
        """
        Returns a response to the data platform

        Returns
        -------
        Response

        """
        data = self._db_manager.get_data(universe=self._universe, fields=self._fields)
        response = Response()
        response.data = data
        return response



import allure
import pytest

from refinitiv.data.content import fundamental_and_reference
from refinitiv.data.errors import RDError
from tests.integration.conftest import set_rdp_config, set_udf_config
from tests.integration.constants_list import HttpStatusCode, HttpReason
from tests.integration.helpers import (
    check_response_dataframe_columns_date_as_index,
    check_index_column_contains_dates,
    check_response_dataframe_universe_date_as_index,
    check_response_dataframe_contains_columns_names,
)
from tests.integration.helpers import (
    get_async_response_from_definition,
    get_async_response_from_definitions,
    check_response_status,
    check_non_empty_response_data,
    check_dataframe_column_date_for_datetime_type,
)

DOW_JONES_CONSTITUENTS = [
    "GS.N",
    "NKE.N",
    "CSCO.OQ",
    "JPM.N",
    "DIS.N",
    "INTC.OQ",
    "DOW.N",
    "MRK.N",
    "CVX.N",
    "AXP.N",
    "VZ.N",
    "HD.N",
    "WBA.OQ",
    "MCD.N",
    "UNH.N",
    "KO.N",
    "JNJ.N",
    "MSFT.OQ",
    "HON.OQ",
    "CRM.N",
    "PG.N",
    "IBM.N",
    "MMM.N",
    "AAPL.OQ",
    "WMT.N",
    "CAT.N",
    "AMGN.OQ",
    "V.N",
    "TRV.N",
    "BA.N",
]


@allure.suite("Content object - Fundamental and Reference")
@allure.feature("Content object - Fundamental and Reference")
@allure.severity(allure.severity_level.CRITICAL)
@pytest.mark.underlying_content("datagrid")
class TestFundamentalAndReference:
    @allure.title(
        "Create fundamental_and_reference definition object with valid params - synchronously"
    )
    @pytest.mark.caseid("37935805")
    @pytest.mark.parametrize(
        "universe,fields,row_headers,expected_column_names",
        [
            ("AMZN.O", "TR.Revenue", ["date"], "Revenue"),
            (
                    ["TRI.N", "IBM.N", "GOOG.O", "AAPL.O"],
                    ["TR.Revenue", "TR.GrossProfit"],
                    "date",
                    ["Revenue", "Gross Profit"],
            ),
        ],
    )
    @pytest.mark.smoke
    def test_fundamental_and_reference_definition_object_with_valid_params_and_get_data(
            self,
            set_underlying_platform_config,
            open_desktop_session,
            universe,
            fields,
            row_headers,
            expected_column_names,
    ):
        response = fundamental_and_reference.Definition(
            universe=universe, fields=fields, row_headers=row_headers
        ).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_response_dataframe_columns_date_as_index(
            response, universe, expected_column_names
        )
        check_response_dataframe_universe_date_as_index(response, universe)

    @allure.title(
        "Create fundamental_and_reference definition object with chain as universe - asynchronously"
    )
    @pytest.mark.caseid("37935806")
    @pytest.mark.parametrize(
        "universe,fields,row_headers,expected_column_names",
        [
            (
                    ["0#.DJI"],
                    ["TR.Revenue", "TR.GrossProfit"],
                    ["date"],
                    ["Revenue", "Gross Profit"],
            ),
            (
                    ["0#.DJI"],
                    ["TR.Revenue", "TR.GrossProfit"],
                    None,
                    ["Revenue", "Gross Profit"],
            ),
        ],
    )
    async def test_fundamental_and_reference_definition_object_with_chain_and_get_data_async(
            self,
            set_underlying_platform_config,
            open_desktop_session_async,
            universe,
            fields,
            row_headers,
            expected_column_names,
    ):
        response = await get_async_response_from_definition(
            fundamental_and_reference.Definition(
                universe=universe, fields=fields, row_headers=row_headers
            )
        )

        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)

        if row_headers is not None:
            check_response_dataframe_columns_date_as_index(
                response, DOW_JONES_CONSTITUENTS, expected_column_names
            )
            check_response_dataframe_universe_date_as_index(
                response, DOW_JONES_CONSTITUENTS
            )
        else:
            check_response_dataframe_contains_columns_names(
                response, expected_column_names
            )

    @allure.title(
        "Create fundamental_and_reference definition object with global/local parameters"
    )
    @pytest.mark.caseid("37935808")
    @pytest.mark.parametrize(
        "universe,fields,parameters,expected_column_names",
        [
            (
                    ["GOOG.O", "AAPL.O"],
                    ["TR.Revenue", "TR.GrossProfit"],
                    {"SDate": "0CY", "Curn": "CAD"},
                    ["Revenue", "Gross Profit"],
            ),
            (
                    ["GOOG.O", "AAPL.O"],
                    ["TR.PriceTargetMean(SDate:0CY)", "TR.LOWPRICE(SDate:0d)"],
                    None,
                    ["Price Target - Mean", "Low Price"],
            ),
        ],
    )
    def test_fundamental_and_reference_definition_object_with_global_or_local_parameters(
            self,
            set_underlying_platform_config,
            open_desktop_session,
            universe,
            fields,
            parameters,
            expected_column_names,
    ):
        response = fundamental_and_reference.Definition(
            universe, fields, parameters
        ).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_response_dataframe_contains_columns_names(response, expected_column_names)

    @allure.title(
        "Create fundamental_and_reference definition object and get timeseries data"
    )
    @pytest.mark.caseid("37935809")
    @pytest.mark.parametrize(
        "universe,fields,parameters,row_headers,expected_column_names",
        [
            (
                    ["GOOG.O", "MSFT.O", "FB.O", "AMZN.O", "TWTR.K"],
                    ["TR.Revenue", "TR.GrossProfit"],
                    {"Scale": 6, "SDate": 0, "EDate": -3, "FRQ": "FY", "Curn": "EUR"},
                    [fundamental_and_reference.RowHeaders.DATE],
                    ["Revenue", "Gross Profit"],
            )
        ],
    )
    def test_fundamental_and_reference_definition_object_and_get_timeseries_data(
            self,
            set_underlying_platform_config,
            open_desktop_session,
            universe,
            fields,
            parameters,
            row_headers,
            expected_column_names,
    ):
        response = fundamental_and_reference.Definition(
            universe, fields, parameters, row_headers
        ).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_response_dataframe_universe_date_as_index(response, universe)
        check_response_dataframe_columns_date_as_index(
            response, universe, expected_column_names
        )
        check_index_column_contains_dates(response)

    @allure.title(
        "Create fundamental_and_reference definition object with invalid params and get error"
    )
    @pytest.mark.caseid("37935811")
    @pytest.mark.parametrize(
        "set_platform_config,universe,fields,expected_error",
        [
            (
                    set_rdp_config,
                    [],
                    ["", "INVALID"],
                    "Error code -1 | No data item defined. Requested universes: []. Requested fields: ['', 'INVALID']",
            ),
            (
                    set_rdp_config,
                    ["INVALID"],
                    ["TR.Revenue", "TR.GrossProfit"],
                    "Error code 412 | Unable to resolve all requested identifiers in ['INVALID'].",
            ),
            (
                    set_rdp_config,
                    ["GOOG.O"],
                    ["TR.INVALID"],
                    "Error code 218 | Unable to resolve all requested fields in ['TR.INVALID']. The formula must contain at least one field or function.",
            ),
            (
                    set_udf_config,
                    [],
                    ["", "INVALID"],
                    "Error code 400 | Validation error Requested universes: []. Requested fields: ['', 'INVALID']",
            ),
            (
                    set_udf_config,
                    ["INVALID"],
                    ["TR.Revenue", "TR.GrossProfit"],
                    "Error code 412 | Unable to resolve all requested identifiers in ['INVALID'].",
            ),
            (
                    set_udf_config,
                    ["GOOG.O"],
                    ["TR.INVALID"],
                    "Error code 218 | Unable to resolve all requested fields in ['TR.INVALID']. The formula must contain at least one field or function.",
            ),
        ],
    )
    def test_fundamental_and_reference_definition_object_with_invalid_params_and_get_error(
            self,
            set_platform_config,
            universe,
            fields,
            expected_error,
            open_desktop_session,
    ):
        set_platform_config("datagrid")
        with pytest.raises(RDError) as error:
            fundamental_and_reference.Definition(
                universe=universe, fields=fields
            ).get_data()

        assert str(error.value) == expected_error

    @allure.title(
        "Create fundamental_and_reference definition object using closed session"
    )
    @pytest.mark.caseid("37935812")
    async def test_fundamental_and_reference_definition_object_using_closed_session(
            self, set_underlying_platform_config, open_desktop_session_async
    ):
        session = open_desktop_session_async
        await session.close_async()
        definition = fundamental_and_reference.Definition([], [])
        with pytest.raises(
                ValueError, match="Session is not opened. Can't send any request"
        ):
            await get_async_response_from_definition(definition)

    @allure.title(
        "Create fundamental_and_reference definition object with extended params"
    )
    @pytest.mark.caseid("37935813")
    @pytest.mark.parametrize(
        "universe,fields,parameters,expected_column_names,extended_params_universe",
        [
            (
                    ["MSFT.O", "FB.O", "AMZN.O", "TWTR.K"],
                    ["TR.Revenue", "TR.Revenue.date", "TR.GrossProfit"],
                    {"Scale": 6, "SDate": 0, "EDate": -3, "FRQ": "FY", "Curn": "EUR"},
                    ["Revenue", "Gross Profit"],
                    ["GOOG.O", "AAPL.O"],
            )
        ],
    )
    def test_fundamental_and_reference_definition_object_with_extended_params(
            self,
            set_underlying_platform_config,
            open_desktop_session,
            universe,
            fields,
            parameters,
            expected_column_names,
            extended_params_universe,
    ):
        extended_params = {"universe": extended_params_universe}
        response = fundamental_and_reference.Definition(
            universe, fields, parameters, extended_params=extended_params
        ).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_response_dataframe_contains_columns_names(response, expected_column_names)
        check_dataframe_column_date_for_datetime_type(response)

    @allure.title(
        "Create fundamental_and_reference definition object with use_field_names_in_headers True"
    )
    @pytest.mark.caseid("38359418")
    @pytest.mark.parametrize(
        "universe,fields,parameters,row_headers,expected_column_names",
        [
            (
                    ["GOOG.O", "MSFT.O", "FB.O", "AMZN.O", "TWTR.K"],
                    ["TR.Revenue", "TR.GrossProfit"],
                    {"Scale": 6, "SDate": 0, "EDate": -3, "FRQ": "FY", "Curn": "EUR"},
                    "date",
                    ["TR.Revenue", "TR.GrossProfit"],
            )
        ],
    )
    def test_fundamental_and_reference_definition_object_with_use_field_names_in_headers_true(
            self,
            set_underlying_platform_config,
            open_desktop_session,
            universe,
            fields,
            parameters,
            row_headers,
            expected_column_names,
    ):
        response = fundamental_and_reference.Definition(
            universe, fields, parameters, row_headers, use_field_names_in_headers=True
        ).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_response_dataframe_columns_date_as_index(
            response, universe, expected_column_names
        )
        check_response_dataframe_universe_date_as_index(response, universe)
        check_index_column_contains_dates(response)

    @allure.title(
        "Create fundamental_and_reference definition object with use mix valid and invalid data and get partial result"
    )
    @pytest.mark.caseid("39506181")
    @pytest.mark.parametrize(
        "universe,fields,expected_columns_names",
        [
            (
                    ["GOOG.O", "INVAL", "FB.O", "AMZN.O", "TWTR.K"],
                    ["TR.GrossProfit", "INVALID_FIELD"],
                    ["Instrument", "TR.GrossProfit"],
            )
        ],
    )
    def test_fundamental_and_reference_definition_object_with_use_mix_valid_and_invalid_data(
            self,
            set_underlying_platform_config,
            open_desktop_session,
            universe,
            fields,
            expected_columns_names,
    ):
        response = fundamental_and_reference.Definition(
            universe, fields, use_field_names_in_headers=True
        ).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_response_dataframe_contains_columns_names(
            response, expected_columns_names
        )

        assert (
                response.errors[0][1]
                == "Unable to collect data for the field 'TR.GrossProfit' and some specific identifier(s)."
        )

    @allure.title(
        "Create async fundamental_and_reference definition object with invalid params and get error"
    )
    @pytest.mark.caseid("37935811")
    @pytest.mark.parametrize(
        "set_platform_config,universe,fields,expected_error",
        [
            (
                    set_rdp_config,
                    [],
                    ["", "INVALID"],
                    "No data item defined. Requested universes: []. Requested fields: ['', 'INVALID']",
            ),
            (
                    set_rdp_config,
                    ["INVALID"],
                    ["TR.Revenue", "TR.GrossProfit"],
                    "Unable to resolve all requested identifiers in ['INVALID'].",
            ),
            (
                    set_rdp_config,
                    ["GOOG.O"],
                    ["TR.INVALID"],
                    "Unable to resolve all requested fields in ['TR.INVALID']. The formula must contain at least one field or function.",
            ),
            (
                    set_udf_config,
                    [],
                    ["", "INVALID"],
                    "Validation error Requested universes: []. Requested fields: ['', 'INVALID']",
            ),
            (
                    set_udf_config,
                    ["INVALID"],
                    ["TR.Revenue", "TR.GrossProfit"],
                    "Unable to resolve all requested identifiers in ['INVALID'].",
            ),
            (
                    set_udf_config,
                    ["GOOG.O"],
                    ["TR.INVALID"],
                    "Unable to resolve all requested fields in ['TR.INVALID']. The formula must contain at least one field or function.",
            ),
        ],
    )
    async def test_async_fundamental_and_reference_definition_object_with_invalid_params_and_get_error(
            self,
            set_platform_config,
            universe,
            fields,
            expected_error,
            open_desktop_session_async,
    ):
        set_platform_config("datagrid")
        task_result = await get_async_response_from_definitions(
            fundamental_and_reference.Definition(universe=universe, fields=fields),
            fundamental_and_reference.Definition(universe=universe, fields=fields),
        )

        for result in task_result:
            for error in result.errors:
                assert expected_error in error, f"Actual error {error}"

    @allure.title(
        "Create fundamental_and_reference definition object with SCREEN to force ticket workflow on UDF"
    )
    @pytest.mark.caseid("")
    @pytest.mark.parametrize(
        "set_platform_config,universe,fields",
        [
            (
                    set_udf_config,
                    [
                        'SCREEN(U(IN(Equity(active,public,private,primary))/*UNV:PublicPrivate*/), Contains(TR.BusinessSummary,"polymer"), CURN=USD)'
                    ],
                    [
                        "TR.CommonName",
                        "TR.HeadquartersCountry",
                        "TR.GICSSector",
                        "TR.OrganizationStatusCode",
                        "TR.Revenue",
                    ],
            )
        ],
    )
    def test_fundamental_and_reference_definition_object_with_screen_universe_get_data(
            self,
            set_platform_config,
            universe,
            fields,
            open_desktop_session,
    ):
        set_platform_config("datagrid")
        response = fundamental_and_reference.Definition(
            universe=universe, fields=fields
        ).get_data()
        assert response.is_success
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)

    @allure.title(
        "Create fundamental_and_reference definition object with SCREEN to force ticket workflow on UDF - asynchronously"
    )
    @pytest.mark.caseid("")
    @pytest.mark.parametrize(
        "set_platform_config,universe,fields",
        [
            (
                    set_udf_config,
                    [
                        'SCREEN(U(IN(Equity(active,public,private,primary))/*UNV:PublicPrivate*/), Contains(TR.BusinessSummary,"polymer"), CURN=USD)'
                    ],
                    [
                        "TR.CommonName",
                        "TR.HeadquartersCountry",
                        "TR.GICSSector",
                        "TR.OrganizationStatusCode",
                        "TR.Revenue",
                    ],
            )
        ],
    )
    async def test_fundamental_and_reference_definition_object_with_screen_universe_get_data_async(
            self,
            set_platform_config,
            universe,
            fields,
            open_desktop_session_async,
    ):
        set_platform_config("datagrid")
        response = await get_async_response_from_definition(
            fundamental_and_reference.Definition(universe=universe, fields=fields)
        )
        assert response.is_success
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)

    @allure.title(
        "Create fundamental_and_reference definition object with Peers and Screen function as universe"
    )
    @pytest.mark.parametrize(
        "universe,fields,row_headers",
        [
            (
                    'Peers("VOD.L")',
                    [
                        "TR.CommonName",
                        "TR.HeadquartersCountry",
                        "TR.GICSSector",
                        "TR.OrganizationStatusCode",
                        "TR.Revenue",
                    ],
                    ["date"],
            ),
            (
                    'SCREEN(U(IN(Equity(active,public,primary))/*UNV:Public*/), IN(TR.HQCountryCode,"AR"), IN(TR.GICSIndustryCode,"401010"))',
                    [
                        "TR.CommonName",
                        "TR.HeadquartersCountry",
                        "TR.GICSSector",
                        "TR.OrganizationStatusCode",
                        "TR.Revenue",
                    ],
                    None,
            ),
            (
                    "VOD.L",
                    [
                        "TR.PeersRank",
                        "TR.RIC",
                        "AVAIL(TR.GrossProfit(Period=LTM,Methodology=InterimSum)/TR.TotalRevenue(Period=LTM,Methodology=InterimSum),TR.GrossProfit(Period=FY0)/TR.TotalRevenue(Period=FY0))*100",
                    ],
                    [fundamental_and_reference.RowHeaders.DATE],
            ),
        ],
    )
    @pytest.mark.caseid("C43855964")
    @pytest.mark.smoke
    def test_fundamental_and_reference_definition_object_with_peers_and_screen_and_get_data(
            self,
            set_underlying_platform_config,
            open_desktop_session,
            universe,
            fields,
            row_headers,
    ):
        response = fundamental_and_reference.Definition(
            universe=universe, fields=fields, row_headers=row_headers
        ).get_data()

        if row_headers:
            check_index_column_contains_dates(response)
        check_non_empty_response_data(response)

        if not row_headers:
            if set_underlying_platform_config == "rdp-underlying-platform":
                links_count = response.data.raw['links']['count']
            else:
                links_count = response.data.raw['totalColumnsCount']

            assert response.data.df.shape[0] == links_count



import pytest

from refinitiv.data.content.ipa.curves import forward_curves
from tests.unit.conftest import load_json


forward_curves_json = load_json("./tests/unit/content/ipa/_curves/forward_curves.json")
zc_curves_json = load_json("./tests/unit/content/ipa/_curves/zc_curves.json")
zc_curve_definitions_json = load_json(
    "./tests/unit/content/ipa/_curves/zc_curve_definitions.json"
)
zc_curves_definitions_json = load_json(
    "./tests/unit/content/ipa/_curves/zc_curves_definitions.json"
)


def create_forward_curve_definition1():
    forward_curve_def = create_forward_curve_def()
    swap_zc_curve_def = create_swap_zc_curve_def()

    definition = forward_curves.Definition(
        curve_definition=swap_zc_curve_def,
        forward_curve_definitions=[forward_curve_def],
        curve_parameters=forward_curves.SwapZcCurveParameters(),
        curve_tag="some_tag",
    )
    return definition


def create_forward_curve_definition2():
    swap_zc_curve_def = create_swap_zc_curve_def()
    forward_curve_def = create_forward_curve_def()

    definition = forward_curves.Definition(swap_zc_curve_def, [forward_curve_def])
    return definition


@pytest.fixture(
    scope="function",
    params=[create_forward_curve_definition1, create_forward_curve_definition2],
)
def forward_curve_definition(request):
    return request.param()


def create_swap_zc_curve_def():
    definition = forward_curves.SwapZcCurveDefinition(
        currency="EUR",
        index_name="EURIBOR",
        discounting_tenor="OIS",
    )
    return definition


def create_forward_curve_def():
    definition = forward_curves.ForwardCurveDefinition(
        index_tenor="3M",
        forward_curve_tag="ForwardTag",
        forward_start_date="2021-02-01",
        forward_curve_tenors=["0D", "1D"],
        forward_start_tenor="some_start_tenor",
    )
    return definition


def create_outputs():
    outputs = forward_curves.Outputs.CONSTITUENTS
    return outputs



from typing import TYPE_CHECKING

from ...._object_definition import ObjectDefinition


if TYPE_CHECKING:
    from ......_types import OptStr


class ValuationTime(ObjectDefinition):
    """
    Generates the Cross Currency curves for the definitions provided

    Parameters
    ----------
    city_name : str, optional
        The city name according to market identifier code (mic) (e.g., 'new york')  see
        iso 10383 for reference.
    local_time : str, optional
        Local time or other words time in offset timezone. the value is expressed in iso
        8601 format: [hh]:[mm]:[ss] (e.g., '14:00:00').
    market_identifier_code : str, optional
        Market identifier code (mic) is a unique identification code used to identify
        securities trading exchanges, regulated and non-regulated trading markets. e.g.
        xnas.  see iso 10383 for reference.
    time_zone_offset : str, optional
        Time offsets from utc. the value is expressed in iso 8601 format: [hh]:[mm]
        (e.g., '+05:00').
    """

    def __init__(
        self,
        city_name: "OptStr" = None,
        local_time: "OptStr" = None,
        market_identifier_code: "OptStr" = None,
        time_zone_offset: "OptStr" = None,
    ) -> None:
        super().__init__()
        self.city_name = city_name
        self.local_time = local_time
        self.market_identifier_code = market_identifier_code
        self.time_zone_offset = time_zone_offset

    @property
    def city_name(self):
        """
        The city name according to market identifier code (mic) (e.g., 'new york')  see
        iso 10383 for reference.
        :return: str
        """
        return self._get_parameter("cityName")

    @city_name.setter
    def city_name(self, value):
        self._set_parameter("cityName", value)

    @property
    def local_time(self):
        """
        Local time or other words time in offset timezone. the value is expressed in iso
        8601 format: [hh]:[mm]:[ss] (e.g., '14:00:00').
        :return: str
        """
        return self._get_parameter("localTime")

    @local_time.setter
    def local_time(self, value):
        self._set_parameter("localTime", value)

    @property
    def market_identifier_code(self):
        """
        Market identifier code (mic) is a unique identification code used to identify
        securities trading exchanges, regulated and non-regulated trading markets. e.g.
        xnas.  see iso 10383 for reference.
        :return: str
        """
        return self._get_parameter("marketIdentifierCode")

    @market_identifier_code.setter
    def market_identifier_code(self, value):
        self._set_parameter("marketIdentifierCode", value)

    @property
    def time_zone_offset(self):
        """
        Time offsets from utc. the value is expressed in iso 8601 format: [hh]:[mm]
        (e.g., '+05:00').
        :return: str
        """
        return self._get_parameter("timeZoneOffset")

    @time_zone_offset.setter
    def time_zone_offset(self, value):
        self._set_parameter("timeZoneOffset", value)



import abc
import asyncio
import itertools
import logging
import warnings
from threading import Lock
from typing import Callable, TYPE_CHECKING, Union
from contextlib import AbstractContextManager, AbstractAsyncContextManager

from . import http_service
from .event_code import EventCode
from ._dacs_params import DacsParams
from ._session_cxn_type import SessionCxnType
from .tools import is_closed
from ... import _configure as configure, _log as log
from ..._open_state import OpenState
from ..._tools import DEBUG, cached_property, create_repr

if TYPE_CHECKING:
    from ._session_cxn_factory import SessionConnection
    from .http_service import HTTPService
    from ..._configure import _RDPConfig
    import httpx
    from ...delivery._data._data_provider import Request

SESSION_IS_CLOSED = "Session is closed"


class Session(AbstractContextManager, AbstractAsyncContextManager):
    _id_iterator = itertools.count()
    # Logger for messages outside of particular session instances

    __acquire_session_id_lock = Lock()

    @staticmethod
    def class_logger():
        return log.create_logger("session")

    @property
    def name(self):
        return self._name

    def __init__(
        self,
        app_key,
        on_state: Callable[[OpenState, str, "Session"], None] = None,
        on_event: Callable[[EventCode, Union[dict, str], "Session"], None] = None,
        token=None,
        deployed_platform_username=None,
        dacs_position=None,
        dacs_application_id=None,
        name="default",
    ):
        with self.__acquire_session_id_lock:
            self._session_id = next(self._id_iterator)

        session_type = self.type.name.lower()
        logger_name = f"sessions.{session_type}.{name}.{self.session_id}"

        self.class_logger().debug(
            f'Creating session "{logger_name}" based on '
            f'session.{session_type}.Definition("{session_type}.{name}")'
        )

        if app_key is None:
            raise ValueError("app_key value can't be None")

        self._state = OpenState.Closed

        self._app_key = app_key
        self._on_state: Callable[[OpenState, str, Session], None] = on_state
        self._on_event: Callable[
            [EventCode, Union[dict, str], Session], None
        ] = on_event
        self._access_token = token
        self._dacs_params = DacsParams()

        if deployed_platform_username:
            self._dacs_params.deployed_platform_username = deployed_platform_username
        if dacs_position:
            self._dacs_params.dacs_position = dacs_position
        if dacs_application_id:
            self._dacs_params.dacs_application_id = dacs_application_id

        self._logger = log.create_logger(logger_name)
        # redirect log method of this object to the log in logger object
        self.log = self._logger.log
        self.warning = self._logger.warning
        self.error = self._logger.error
        self.debug = self._logger.debug
        self.info = self._logger.info

        self._name = name
        self._config: "_RDPConfig" = configure.get_config().copy()

        # override session api config with session's specific api parameters
        specific_api_path = f"sessions.{session_type}.{name}.apis"
        specific_api = self._config.get(specific_api_path)
        if specific_api:
            self._config.set_param("apis", specific_api)

        self._config.on(configure.ConfigEvent.UPDATE, self._on_config_updated)
        # rssl/rwf stream ids always starts with 5
        self._omm_stream_counter = itertools.count(5)

        self._rdp_stream_counter = itertools.count(5)  # can not be 0
        self._contrib_post_id_counter = itertools.count(5)

    def __enter__(self):
        self.open()
        return self

    def __exit__(self, exc_type, exc_value, exc_traceback):
        self.close()

    async def __aenter__(self):
        await self.open_async()
        return self

    async def __aexit__(self, exc_type, exc_value, traceback):
        await self.close_async()

    @cached_property
    def _http_service(self) -> "HTTPService":
        return http_service.get_service(self)

    @cached_property
    def _connection(self) -> "SessionConnection":
        from ._session_cxn_factory import get_session_cxn

        cxn_type = self._get_session_cxn_type()
        cxn = get_session_cxn(cxn_type, self)
        self.debug(f"Created session connection {cxn_type}")
        return cxn

    @abc.abstractmethod
    def _get_session_cxn_type(self) -> SessionCxnType:
        pass

    def on_state(self, callback: Callable[[OpenState, str, "Session"], None]) -> None:
        """
        Registers a defined callback function.
        Called by the library whenever the state of the session is updated.

        Parameters
        ----------
        callback: Callable[[OpenState, str, Session], None]
            Callback function or method to be notified of any state change.

        Raises
        ----------
        TypeError
            If user provided invalid object type
        """
        if not callable(callback):
            raise TypeError("Please provide callable object")

        self._on_state = callback

    def _call_on_state(self, message: str):
        if not self._on_state:
            return
        self.debug(f"Session calls on_state({self}, {self._state}, {message})")
        try:
            self._on_state(self._state, message, self)
        except Exception as e:
            self.error(
                f"on_state user function on session {self.session_id} raised error {e}"
            )

    def on_event(
        self, callback: Callable[[EventCode, Union[dict, str], "Session"], None]
    ) -> None:
        """
        Registers a defined callback function.
        Called by the library whenever the event of the session is updated.

        Parameters
        ----------
        callback: Callable[[EventCode, Union[dict, str], "Session"], None]
            Callback function or method to be notified of any event change.

        Raises
        ----------
        TypeError
            If user provided invalid object type
        """
        if not callable(callback):
            raise TypeError("Please provide callable object")

        self._on_event = callback

    def _call_on_event(self, event: EventCode, message: Union[dict, str]):
        if not self._on_event:
            return
        self.debug(f"Session calls on_event({self}, {event}, {message})")
        try:
            self._on_event(event, message, self)
        except Exception as e:
            self.error(
                f"on_event user function on session {self.session_id} raised error {e}"
            )

    def __repr__(self):
        return create_repr(
            self,
            middle_path="session",
            content=f"{{name='{self.name}'}}",
        )

    def _on_config_updated(self):
        log_level = log.read_log_level_config()

        if log_level != self.get_log_level():
            self.set_log_level(log_level)

    @property
    def config(self) -> "_RDPConfig":
        return self._config

    @property
    def open_state(self):
        """
        Returns the session state.
        """
        return self._state

    @property
    def app_key(self):
        """
        Returns the application id.
        """
        return self._app_key

    @app_key.setter
    def app_key(self, app_key):
        """
        Set the application key.
        """
        from ...eikon._tools import is_string_type

        if app_key is None:
            return
        if not is_string_type(app_key):
            raise AttributeError("application key must be a string")

        self._app_key = app_key

    def update_access_token(self, access_token):
        DEBUG and self.debug(
            f"Session.update_access_token(access_token='{access_token}'"
        )
        self._access_token = access_token

        from ...delivery._stream import stream_cxn_cache

        if stream_cxn_cache.has_cxns(self):
            cxns_by_session = stream_cxn_cache.get_cxns(self)
            for cxn in cxns_by_session:
                cxn.send_login_message()

    @property
    def session_id(self):
        return self._session_id

    def logger(self) -> logging.Logger:
        return self._logger

    def _get_rdp_url_root(self) -> str:
        return ""

    @cached_property
    def http_request_timeout_secs(self):
        return self._http_service.request_timeout_secs

    ############################################################
    #   reconnection configuration

    @property
    def stream_auto_reconnection(self):
        return True

    @property
    def server_mode(self):
        return False

    @abc.abstractmethod
    def get_omm_login_message(self):
        """return the login message for OMM 'key' section"""
        pass

    @abc.abstractmethod
    def get_rdp_login_message(self, stream_id):
        """return the login message for RDP protocol"""
        pass

    ######################################
    # methods to manage log              #
    ######################################
    def set_log_level(self, log_level: [int, str]) -> None:
        """
        Sets the level of log messages.
        By default, logs are disabled.

        Parameters
        ----------
        log_level : int | str
            Log level value. Possible values are :
            [CRITICAL, FATAL, ERROR, WARNING, WARN, INFO, DEBUG, NOTSET]
        """
        log_level = log.convert_log_level(log_level)
        self._logger.setLevel(log_level)

        if DEBUG:
            # Enable debugging

            # Report all mistakes managing asynchronous resources.
            warnings.simplefilter("always", ResourceWarning)

    def get_log_level(self):
        """
        Gets the current log level.

        Returns
        -------
        level
        """
        return self._logger.level

    def trace(self, message):
        self._logger.log(log.TRACE, message)

    ######################################
    # methods to open and close session  #
    ######################################
    def open(self) -> OpenState:
        """
        Opens the session.

        Returns
        -------
        OpenState
        """
        if is_closed(self):
            self.debug(f"Open session")

            self._state = OpenState.Pending
            self._call_on_state("Session opening in progress")
            self._config.on(configure.ConfigEvent.UPDATE, self._on_config_updated)
            self._http_service.open()
            is_opened = self._connection.open()

            if is_opened:
                self._state = OpenState.Opened
                self._call_on_state("Session is opened")
                self.debug(f"Opened session")
            else:
                self.close()
                self._state = OpenState.Closed
                self._call_on_state(SESSION_IS_CLOSED)

        return self._state

    async def open_async(self) -> OpenState:
        """
        Opens the session asynchronously.

        Returns
        -------
        OpenState
        """
        if is_closed(self):
            self.debug(f"Open async session")

            self._state = OpenState.Pending
            self._call_on_state("Session opening in progress")
            self._config.on(configure.ConfigEvent.UPDATE, self._on_config_updated)
            await self._http_service.open_async()
            is_opened = self._connection.open()

            if is_opened:
                self._state = OpenState.Opened
                self._call_on_state("Session is opened")
            else:
                await self.close_async()
                self._state = OpenState.Closed
                self._call_on_state(SESSION_IS_CLOSED)

            self.debug(f"Opened async session")

        return self._state

    def close(self) -> OpenState:
        """
        Closes the session.

        Returns
        -------
        OpenState
        """
        if not is_closed(self):
            self.debug(f"Close session")

            self._state = OpenState.Closed

            from ...delivery._stream import stream_cxn_cache

            stream_cxn_cache.close_cxns(self)
            self._http_service.close()
            self._connection.close()

            if DEBUG:
                import time
                from ...delivery._stream import stream_cxn_cache
                import threading

                time.sleep(5)
                s = "\n\t".join([str(t) for t in threading.enumerate()])
                self.debug(f"Threads:\n\t{s}")

                if stream_cxn_cache.has_cxns(self):
                    raise AssertionError(
                        f"Not all cxns are closed (session={self},\n"
                        f"cxns={stream_cxn_cache.get_cxns(self)})"
                    )

            self._config.remove_listener(
                configure.ConfigEvent.UPDATE, self._on_config_updated
            )
            self._call_on_state(SESSION_IS_CLOSED)
            self.debug(f"Closed session")

        return self._state

    async def close_async(self) -> OpenState:
        """
        Closes the session asynchronously.

        Returns
        -------
        OpenState
        """
        if not is_closed(self):
            self.debug(f"Close async session")

            self._state = OpenState.Closed

            from ...delivery._stream import stream_cxn_cache

            stream_cxn_cache.close_cxns(self)
            await self._http_service.close_async()
            self._connection.close()

            if DEBUG:
                from ...delivery._stream import stream_cxn_cache
                import threading

                await asyncio.sleep(5)
                s = "\n\t".join([str(t) for t in threading.enumerate()])
                self.debug(f"Threads:\n\t{s}")

                if stream_cxn_cache.has_cxns(self):
                    raise AssertionError(
                        f"Not all cxns are closed (session={self},\n"
                        f"cxns={stream_cxn_cache.get_cxns(self)})"
                    )

            self._config.remove_listener(
                configure.ConfigEvent.UPDATE, self._on_config_updated
            )
            self._call_on_state(SESSION_IS_CLOSED)
            self.debug(f"Closed async session")

        return self._state

    async def http_request_async(self, request: "Request") -> "httpx.Response":
        return await self._http_service.request_async(request)

    def http_request(self, request: "Request") -> "httpx.Response":
        return self._http_service.request(request)

    def verify_scope(self, key: str, method: str):
        pass



from refinitiv.data.content.ipa._curves._cross_currency_curves._definitions import (
    CrossCurrencyInstrumentDefinition,
)


def test_cross_currency_instrument_definition():
    # given
    expected_dict = {
        "instrumentCode": "string",
        "tenor": "string",
        "isNonDeliverable": True,
        "syntheticInstrumentCode": "string",
        "template": "string",
    }

    # when
    testing_obj = CrossCurrencyInstrumentDefinition(
        instrument_code="string",
        tenor="string",
        is_non_deliverable=True,
        synthetic_instrument_code="string",
        template="string",
    )

    # then
    assert testing_obj.get_dict() == expected_dict

    assert testing_obj.instrument_code == "string"
    assert testing_obj.tenor == "string"
    assert testing_obj.is_non_deliverable is True
    assert testing_obj.synthetic_instrument_code == "string"
    assert testing_obj.template == "string"



import allure
import pytest

from refinitiv.data.content.ownership.fund import concentration
from refinitiv.data.errors import RDError
from tests.integration.constants_list import HttpStatusCode, HttpReason
from tests.integration.content.ownership.helpers import (
    check_response_dataframe_instrument_contains_expected_universes,
    get_expected_column,
)
from tests.integration.helpers import (
    get_async_response_from_definitions,
    check_response_status,
    check_non_empty_response_data,
    check_response_dataframe_contains_columns_names,
    check_extended_params_were_sent_in_request,
)

EXPECTED_COLUMN_NAMES = [
    "instrument",
    "Investors Total",
    "% SharesOutstandingAll",
    "SharesHeldAll",
    "SharesHeld ValueAll",
]

EXPECTED_COLUMN_TITLES = [
    "Instrument",
    "Investors Total",
    "% SharesOutstandingAll",
    "SharesHeldAll",
    "SharesHeld ValueAll",
]


@allure.suite("Content object - Ownership Fund Concentration")
@allure.feature("Content object - Ownership Fund Concentration")
@allure.severity(allure.severity_level.CRITICAL)
class TestFundConcentration:
    @allure.title(
        "Create Fund Concentration definition object with valid required params and get data"
    )
    @pytest.mark.caseid("37478856")
    @pytest.mark.parametrize(
        "universe,use_field_names_in_headers,extended_params",
        [
            ("US02079K1079", True, {"test_param": "test_value"}),
            (["ONEX.CCP", "TRI.N", "invalid universe"], False, None),
        ],
    )
    def test_create_fund_concentration_object_with_valid_fields_and_get_data(
        self,
        open_platform_session,
        universe,
        use_field_names_in_headers,
        extended_params,
    ):
        expected_column = get_expected_column(
            use_field_names_in_headers, EXPECTED_COLUMN_TITLES, EXPECTED_COLUMN_NAMES
        )
        response = concentration.Definition(
            universe, use_field_names_in_headers, extended_params
        ).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_response_dataframe_contains_columns_names(response, expected_column)
        check_response_dataframe_instrument_contains_expected_universes(
            response, universe
        )
        check_extended_params_were_sent_in_request(response, extended_params)

    @allure.title(
        "Create Fund Concentration definition object with mix valid and invalid params and get data asynchronously"
    )
    @pytest.mark.caseid("37478857")
    @pytest.mark.parametrize(
        "universe,invalid_universe",
        [
            (["ONEX.CCP", "invalid universe"], "invalid universe"),
        ],
    )
    @pytest.mark.smoke
    async def test_create_fund_concentration_object_with_mix_valid_and_invalid_params_and_get_data_async(
        self, open_platform_session_async, universe, invalid_universe
    ):
        valid_response, invalid_response = await get_async_response_from_definitions(
            concentration.Definition(universe, use_field_names_in_headers=True),
            concentration.Definition(invalid_universe, use_field_names_in_headers=True),
        )
        check_response_status(
            response=valid_response,
            expected_status_code=HttpStatusCode.TWO_HUNDRED,
            expected_http_reason=HttpReason.OK,
            expected_error_code=None,
            expected_error_message="Failed to resolve identifiers ['invalid universe']",
        )
        check_non_empty_response_data(valid_response)
        check_response_status(
            response=invalid_response,
            expected_status_code=HttpStatusCode.TWO_HUNDRED,
            expected_http_reason=HttpReason.OK,
            expected_error_code=HttpStatusCode.FOUR_HUNDRED_TWELVE,
            expected_error_message="Unable to resolve all requested identifiers. Requested items: ['invalid universe']",
        )
        check_response_dataframe_contains_columns_names(
            valid_response, EXPECTED_COLUMN_NAMES
        )
        check_response_dataframe_instrument_contains_expected_universes(
            valid_response, universe
        )

    @allure.title(
        "Create Fund Concentration definition object with invalid universe and get error"
    )
    @pytest.mark.caseid("37478858")
    @pytest.mark.parametrize(
        "universe",
        [[], [""]],
    )
    def test_create_fund_concentration_object_with_invalid_universe_and_get_error(
        self, open_platform_session, universe
    ):
        with pytest.raises(RDError) as error:
            concentration.Definition(universe).get_data()
        assert (
            str(error.value)
            == "Error code 400 | Validation error: validation failure list:\nuniverse.0 in query should be at least 1 chars long"
        )



from unittest.mock import Mock

import refinitiv.data.content.ipa.financial_contracts as rdf
from refinitiv.data.content._content_data import Data
from refinitiv.data.delivery._data._data_factory import DataFactory
from refinitiv.data.delivery._data._data_provider import Response, ParsedData
from tests.unit.conftest import StubSession


def test_option_stream_definition_with_fields():
    # given
    session = StubSession(is_open=True)
    expected_fields = ["BID", "ASK"]

    # when
    definition = rdf.option.Definition(fields=expected_fields)
    definition.get_stream(session=session)

    # then
    assert definition._kwargs["fields"] == expected_fields


def test_option_stream_definition_without_fields():
    # given
    session = StubSession(is_open=True)

    # when
    definition = rdf.option.Definition()
    definition.get_stream(session=session)

    # then
    assert definition._kwargs["fields"] == [], str(definition._kwargs["fields"])


def test_option_stream_definition_without_fields_mock_response():
    # given
    path_response = Response(
        True,
        ParsedData({}, {}, {"headers": [{"name": "ASK"}, {"name": "IBM"}]}),
        DataFactory(),
    )

    session = StubSession(is_open=True)
    definition = rdf.option.Definition()
    definition.get_data = Mock(return_value=path_response)

    # when
    definition.get_stream(session=session)

    # then
    assert definition._kwargs["fields"] == ["ASK", "IBM"]



__all__ = (
    "consolidated",
    "Frequency",
    "fund",
    "insider",
    "investor",
    "org_info",
    "SortOrder",
    "StatTypes",
)

from ._enums import SortOrder, StatTypes, Frequency
from . import consolidated, fund, investor, insider, org_info



import types
from datetime import datetime
from functools import partial

import pytest

import refinitiv.data as rd
from refinitiv.data.delivery._data._request import Request
from tests.unit.conftest import StubSession, StubResponse
from .test_enums import get_json, get_query_params_from_url

items = [
    rd.get_history,
    rd.content.news.headlines.Definition,
    rd.content.custom_instruments.summaries.Definition,
    rd.content.custom_instruments.events.Definition,
    rd.content.historical_pricing.events.Definition,
    rd.content.historical_pricing.summaries.Definition,
    rd.content.ownership.fund.shareholders_history_report.Definition,
    rd.content.ownership.consolidated.shareholders_history_report.Definition,
    rd.content.ownership.insider.transaction_report.Definition,
    rd.dates_and_calendars.add_periods,
    rd.dates_and_calendars.count_periods,
    rd.dates_and_calendars.date_schedule,
    rd.dates_and_calendars.is_working_day,
    rd.dates_and_calendars.holidays,
    rd.content.ipa.dates_and_calendars.add_periods.Definition,
    rd.content.ipa.dates_and_calendars.count_periods.Definition,
    rd.content.ipa.dates_and_calendars.date_schedule.Definition,
    rd.content.ipa.dates_and_calendars.is_working_day.Definition,
    rd.content.ipa.dates_and_calendars.holidays.Definition,
    rd._qpl.fx_swp_to_swp,
]


def test_datetime_arguments():
    for item in items:
        if not isinstance(item, types.FunctionType):
            item = getattr(item, "__init__")
        annotations = item.__annotations__
        if not annotations:
            continue

        has = False
        for param_name, type_notation in annotations.items():
            if type_notation == "OptDateTime":
                has = True
                break

        assert has, item


def make_checker(param_name, value, func_get_params, mocked_json):
    def http_request(request: Request):
        params = func_get_params(request)

        if isinstance(params, list):
            params = params[0]

        assert params[param_name] == str(value), (
            f"Param{param_name}:\n"
            f"Expected value {params[param_name]}. \n"
            f"Value {str(value)}"
        )
        return StubResponse(mocked_json=mocked_json)

    return http_request


@pytest.mark.xfail
@pytest.mark.parametrize(
    argnames=(
        "definition",
        "date",
        "date_arg_name",
        "request_param_name",
        "func_get_params",
        "mocked_json",
    ),
    argvalues=[
        (
            rd.content.news.headlines.Definition(
                query="Refinitiv", count=10, date_from=datetime(2020, 1, 1)
            ),
            "2020-01-01T00%3A00%3A00",
            "date_from",
            "dateFrom",
            get_query_params_from_url,
            None,
        ),
        (
            rd.content.ipa.dates_and_calendars.add_periods.Definition(
                start_date=datetime(2020, 1, 1), period="1Y"
            ),
            "2020-01-01",
            "start_date",
            "startDate",
            get_json,
            [{"data": [{}], "headers": [], "error": []}],
        ),
        (
            rd.content.ipa.dates_and_calendars.count_periods.Definition(
                start_date=datetime(2020, 1, 1), end_date="2022-01-01"
            ),
            "2020-01-01",
            "start_date",
            "startDate",
            get_json,
            [{"data": [{}], "headers": [], "error": []}],
        ),
        (
            rd.content.ipa.dates_and_calendars.date_schedule.Definition(
                start_date=datetime(2020, 1, 1), frequency="Daily"
            ),
            "2020-01-01",
            "start_date",
            "startDate",
            get_json,
            [{"data": [{}], "headers": [], "error": []}],
        ),
        (
            rd.content.ipa.dates_and_calendars.is_working_day.Definition(
                date=datetime(2020, 1, 1)
            ),
            "2020-01-01",
            "date",
            "date",
            get_json,
            [{"data": [{}], "headers": [], "error": []}],
        ),
        (
            rd.content.ipa.dates_and_calendars.holidays.Definition(
                start_date=datetime(2020, 1, 1), end_date="2022-01-01"
            ),
            "2020-01-01",
            "date",
            "date",
            get_json,
            [{"data": [{}], "headers": [], "error": []}],
        ),
        (
            rd.content.custom_instruments.summaries.Definition(
                universe=["EUR"], start=datetime(2020, 1, 1)
            ),
            "2020-01-01",
            "start",
            "start",
            get_json,
            None,
        ),
        (
            rd.content.custom_instruments.events.Definition(
                universe=["EUR"], start=datetime(2020, 1, 1)
            ),
            "2020-01-01",
            "start",
            "start",
            get_json,
            None,
        ),
        (
            rd.content.historical_pricing.events.Definition(
                universe=["EUR"], start=datetime(2020, 1, 1)
            ),
            "2020-01-01T00%3A00%3A00.000000000Z",
            "start",
            "start",
            get_query_params_from_url,
            None,
        ),
        (
            rd.content.historical_pricing.summaries.Definition(
                universe=["EUR"], start=datetime(2020, 1, 1)
            ),
            "2020-01-01T00%3A00%3A00.000000000Z",
            "start",
            "start",
            get_query_params_from_url,
            None,
        ),
        (
            rd.content.ownership.fund.shareholders_history_report.Definition(
                universe="TRI.N", frequency="Q", start=datetime(2020, 1, 1)
            ),
            "2020-01-01T00%3A00%3A00.000000000Z",
            "start",
            "start",
            get_query_params_from_url,
            None,
        ),
        (
            rd.content.ownership.consolidated.shareholders_history_report.Definition(
                universe="TRI.N", frequency="Q", start=datetime(2020, 1, 1)
            ),
            "2020-01-01T00%3A00%3A00.000000000Z",
            "start",
            "start",
            get_query_params_from_url,
            None,
        ),
        (
            rd.content.ownership.insider.transaction_report.Definition(
                universe="TRI.N", start=datetime(2020, 1, 1)
            ),
            "2020-01-01T00%3A00%3A00.000000000Z",
            "start",
            "start",
            get_query_params_from_url,
            None,
        ),
    ],
    ids=(
        "rd.content.news.headlines.Definition",
        "rd.content.ipa.dates_and_calendars.add_periods.Definition",
        "rd.content.ipa.dates_and_calendars.count_periods.Definition",
        "rd.content.ipa.dates_and_calendars.date_schedule.Definition",
        "rd.content.ipa.dates_and_calendars.is_working_day.Definition",
        "rd.content.ipa.dates_and_calendars.holidays.Definition",
        "rd.content.custom_instruments.summaries.Definition",
        "rd.content.custom_instruments.events.Definition",
        "rd.content.historical_pricing.events.Definition",
        "rd.content.historical_pricing.summaries.Definition",
        "rd.content.ownership.fund.shareholders_history_report.Definition",
        "rd.content.ownership.consolidated.shareholders_history_report.Definition",
        "rd.content.ownership.insider.transaction_report.Definition",
    ),
)
def test_datetime_in_definition(
    definition, date, date_arg_name, request_param_name, func_get_params, mocked_json
):
    session = StubSession(is_open=True)
    session.config.set_param("apis.data.news.underlying-platform", "rdp")

    session.http_request = make_checker(
        request_param_name, date, func_get_params, mocked_json
    )

    try:
        definition.get_data(session=session)
    except (KeyError, IndexError):
        pass


@pytest.mark.parametrize(
    (
        "function",
        "date",
        "date_arg_name",
        "request_param_name",
        "func_get_params",
        "mocked_json",
    ),
    [
        (
            partial(
                rd._qpl.fx_swp_to_swp,
                fx_cross_code="code",
                market_data_date_time=datetime(2020, 1, 1),
            ),
            "2020-01-01",
            "market_data_date_time",
            "marketDataDate",
            get_json,
            None,
        ),
    ],
)
def test_datetime_in_function(
    function, date, date_arg_name, request_param_name, func_get_params, mocked_json
):
    session = StubSession(is_open=True)
    rd.session.set_default(session)
    session.http_request = make_checker(
        request_param_name, date, func_get_params, mocked_json
    )

    try:
        function()
    except (KeyError, IndexError):
        pass

    rd.session.set_default(None)



import pytest
from mock import MagicMock
from refinitiv.data.session import platform
from refinitiv.data._core.session import GrantPassword


def test_definition_keyword_arguments():
    app_key_mock = MagicMock(name="MOCK-app_key")
    grant_mock = GrantPassword(
        MagicMock(name="MOCK-username"), MagicMock(name="MOCK-password")
    )
    signon_control_mock = MagicMock(name="MOCK-signon_control")
    deployed_platform_host_mock = MagicMock(name="MOCK-deployed_platform_host")
    deployed_platform_username_mock = MagicMock(name="MOCK-deployed_platform_username")
    dacs_position_mock = MagicMock(name="MOCK-dacs_position")
    dacs_application_id_mock = MagicMock(name="MOCK-dacs_application_id")

    definition = platform.Definition(
        grant=grant_mock,
        app_key=app_key_mock,
        signon_control=signon_control_mock,
        deployed_platform_host=deployed_platform_host_mock,
        deployed_platform_username=deployed_platform_username_mock,
        dacs_position=dacs_position_mock,
        dacs_application_id=dacs_application_id_mock,
    )
    session = definition.get_session()

    assert session.app_key == app_key_mock
    assert session._grant is grant_mock
    assert session._grant.get_username() == grant_mock.get_username()
    assert session._grant.get_password() == grant_mock.get_password()
    assert session._take_signon_control == signon_control_mock
    assert session._deployed_platform_host == deployed_platform_host_mock

    assert session.name == "default"
    assert (
        session._dacs_params.deployed_platform_username
        == deployed_platform_username_mock
    )
    assert session._dacs_params.dacs_position == dacs_position_mock
    assert session._dacs_params.dacs_application_id == dacs_application_id_mock


def test_definition_positional_arguments():
    session_name = "default"
    app_key_mock = MagicMock(name="MOCK-app_key")
    grant_mock = GrantPassword(
        MagicMock(name="MOCK-username"), MagicMock(name="MOCK-password")
    )
    signon_control_mock = MagicMock(name="MOCK-signon_control")
    deployed_platform_host_mock = MagicMock(name="MOCK-deployed_platform_host")
    authentication_token_mock = MagicMock(name="MOCK-authentication_token")
    deployed_platform_username_mock = MagicMock(name="MOCK-deployed_platform_username")
    dacs_position_mock = MagicMock(name="MOCK-dacs_position")
    dacs_application_id_mock = MagicMock(name="MOCK-dacs_application_id")

    definition = platform.Definition(
        session_name,
        app_key_mock,
        grant_mock,
        signon_control_mock,
        deployed_platform_host_mock,
        deployed_platform_username_mock,
        dacs_position_mock,
        dacs_application_id_mock,
    )
    session = definition.get_session()

    assert session.name == session_name
    assert session.app_key == app_key_mock
    assert session._grant is grant_mock
    assert session._grant.get_username() == grant_mock.get_username()
    assert session._grant.get_password() == grant_mock.get_password()
    assert session._take_signon_control == signon_control_mock
    assert session._deployed_platform_host == deployed_platform_host_mock
    assert session.name == "default"
    assert not authentication_token_mock.called
    assert (
        session._dacs_params.deployed_platform_username
        == deployed_platform_username_mock
    )
    assert session._dacs_params.dacs_position == dacs_position_mock
    assert session._dacs_params.dacs_application_id == dacs_application_id_mock


def test_definition_mix_of_positional_and_named_arguments():
    session_name = "default"
    app_key_mock = MagicMock(name="MOCK-app_key")
    grant_mock = GrantPassword(
        MagicMock(name="MOCK-username"), MagicMock(name="MOCK-password")
    )
    signon_control_mock = MagicMock(name="MOCK-signon_control")
    deployed_platform_host_mock = MagicMock(name="MOCK-deployed_platform_host")
    authentication_token_mock = MagicMock(name="MOCK-authentication_token")
    deployed_platform_username_mock = MagicMock(name="MOCK-deployed_platform_username")
    dacs_position_mock = MagicMock(name="MOCK-dacs_position")
    dacs_application_id_mock = MagicMock(name="MOCK-dacs_application_id")

    definition = platform.Definition(
        session_name,
        app_key_mock,
        grant_mock,
        signon_control_mock,
        deployed_platform_host=deployed_platform_host_mock,
        deployed_platform_username=deployed_platform_username_mock,
        dacs_position=dacs_position_mock,
        dacs_application_id=dacs_application_id_mock,
    )
    session = definition.get_session()

    assert session.app_key == app_key_mock
    assert session._grant is grant_mock
    assert session._grant.get_username() == grant_mock.get_username()
    assert session._grant.get_password() == grant_mock.get_password()
    assert session._take_signon_control == signon_control_mock
    assert session._deployed_platform_host == deployed_platform_host_mock
    assert session.name == "default"
    assert not authentication_token_mock.called
    assert (
        session._dacs_params.deployed_platform_username
        == deployed_platform_username_mock
    )
    assert session._dacs_params.dacs_position == dacs_position_mock
    assert session._dacs_params.dacs_application_id == dacs_application_id_mock


def test_deployed_platform_session():
    session_name = "default"
    app_key_mock = "MOCK-app_key"
    deployed_platform_host_mock = "MOCK-deployed_platform_host"
    deployed_platform_username_mock = "MOCK-deployed_platform_username"

    definition = platform.Definition(
        session_name,
        app_key_mock,
        deployed_platform_host=deployed_platform_host_mock,
        deployed_platform_username=deployed_platform_username_mock,
    )
    session = definition.get_session()
    assert session.app_key == app_key_mock
    assert session._deployed_platform_host == deployed_platform_host_mock
    assert session.name == "default"
    assert (
        session._dacs_params.deployed_platform_username
        == deployed_platform_username_mock
    )


def test_deployed_platform_session_with_missing_argument():
    session_name = "default"
    app_key_mock = MagicMock(name="MOCK-app_key")
    deployed_platform_host_mock = MagicMock(name="MOCK-deployed_platform_host")
    msg = (
        "To create platform session, please provide 'grant' attribute to the "
        "definition or set 'username' and 'password' in the config file. "
        "To create deployed session, please provide 'deployed_platform_host' "
        "and 'deployed_platform_username' to the definition or the config file."
    )

    with pytest.raises(
        AttributeError,
        match=msg,
    ):
        definition = platform.Definition(
            session_name,
            app_key_mock,
            deployed_platform_host=deployed_platform_host_mock,
        )
        definition.get_session()


def test_app_key_raise_error_if_empty_string():
    with pytest.raises(Exception) as error:
        platform.Definition(app_key="")

    assert "Can't find 'app-key' in config object." in str(error.value)


def test_app_key_raise_error_if_none():
    msg = (
        "Please, set app-key in [session.platform.default] section of "
        "the config file or provide 'app-key' attribute to the definition."
    )
    with pytest.raises(Exception) as error:
        platform.Definition(app_key=None)

    assert msg in str(error.value)


def test_signon_control_default_value():
    grant_mock = GrantPassword(
        MagicMock(name="MOCK-username"), MagicMock(name="MOCK-password")
    )

    # when
    session = platform.Definition(app_key="app_key", grant=grant_mock).get_session()

    # then
    assert session._take_signon_control is True


@pytest.mark.parametrize(
    "input_value, expected_value",
    [
        (True, True),
        (False, False),
    ],
)
def test_signon_control(input_value, expected_value):
    grant_mock = GrantPassword(
        MagicMock(name="MOCK-username"), MagicMock(name="MOCK-password")
    )

    # when
    session = platform.Definition(
        app_key="app_key", grant=grant_mock, signon_control=input_value
    ).get_session()

    # then
    assert session._take_signon_control is expected_value



from typing import Type, TYPE_CHECKING

from ._content_data import Data
from ..delivery._data._response_factory import ResponseFactory

if TYPE_CHECKING:
    from ..delivery._data._endpoint_data import ABCData
    from ..delivery._data._response import Response
    from ..delivery._data._parsed_data import ParsedData


class ContentResponseFactory(ResponseFactory):
    def __init__(
        self,
        response_class: Type["Response"] = None,
        data_class: Type["ABCData"] = None,
    ):
        data_class = data_class or Data
        super().__init__(response_class, data_class)

    @staticmethod
    def get_dfbuilder(content_type=None, dfbuild_type=None, **kwargs):
        from ._df_builder_factory import get_dfbuilder, DFBuildType
        from .._content_type import ContentType

        content_type = content_type or kwargs.get(
            "__content_type__", ContentType.DEFAULT
        )
        dfbuild_type = dfbuild_type or kwargs.get(
            "__dfbuild_type__", DFBuildType.DATE_AS_INDEX
        )
        return get_dfbuilder(content_type, dfbuild_type)

    def create_data_success(self, parsed_data: "ParsedData", **kwargs) -> Data:
        return self.data_class(
            self.get_raw(parsed_data),
            self.get_dfbuilder(**kwargs),
            **kwargs,
        )

    def create_data_fail(self, parsed_data: "ParsedData", **kwargs) -> Data:
        return self.data_class(
            parsed_data.content_data or {},
            self.get_dfbuilder(**kwargs),
            **kwargs,
        )



from types import SimpleNamespace


class StubDefinition:
    def __init__(self, axis) -> None:
        request_item = SimpleNamespace()
        surface_parameters = SimpleNamespace()
        surface_parameters._get_enum_parameter = lambda *args: axis
        request_item.surface_parameters = surface_parameters

        self._kwargs = {"universe": request_item}



from ._underlying_type import UnderlyingType
from ._settlement_type import SettlementType
from ._fx_binary_type import FxBinaryType
from ._double_binary_type import DoubleBinaryType
from ...._enums import (
    AverageType,
    BarrierMode,
    BarrierStyle,
    BinaryType,
    BuySell,
    CallPut,
    ExerciseStyle,
    FixingFrequency,
    FxSwapCalculationMethod,
    InOrOut,
    OptionVolatilityType,
    PremiumSettlementType,
    PriceSide,
    PricingModelType,
    Status,
    TimeStamp,
    UpOrDown,
    VolatilityModel,
    VolatilityType,
)



from typing import Iterable, Union, Optional

from ._enums import ZcCurvesOutputs
from ._zc_curve_definitions import ZcCurveDefinitions
from ._zc_curve_parameters import ZcCurveParameters
from ._models import (
    Step,
    Turn,
    Constituents,
)
from ..curves import zc_curves
from ...._types import Strings

Steps = Union[Iterable[Step]]
Turns = Union[Iterable[Turn]]
OptConstituents = Optional[Constituents]
CurveDefinition = Optional[ZcCurveDefinitions]
CurveParameters = Optional[ZcCurveParameters]
DefnDefns = Union[zc_curves.Definition, Iterable[zc_curves.Definition]]
Outputs = Union[Strings, Iterable[ZcCurvesOutputs]]



import abc
from itertools import groupby, zip_longest
from typing import TYPE_CHECKING, List

from ._intervals_consts import NON_INTRA_DAY_INTERVALS
from .._tools._dataframe import convert_dtypes
from .._tools import ohlc

if TYPE_CHECKING:
    from .context_collection import ADCContext, CustInstContext, HPContext
    from ._containers import FieldsContainer, UniverseContainer
    from ..content._df_builder import DFBuilder


class HistoryProvider(abc.ABC):
    @property
    @abc.abstractmethod
    def dfbuilder(self) -> "DFBuilder":
        #  for override
        pass

    @property
    @abc.abstractmethod
    def date_name(self) -> str:
        #  for override
        pass

    def get_df(
        self,
        adc: "ADCContext",
        hp: "HPContext",
        cust_inst: "CustInstContext",
        universe: "UniverseContainer",
        fields: "FieldsContainer",
        interval: str,
        use_field_names_in_headers: bool,
    ):
        if adc.can_build_df:
            return adc.build_df()

        if hp.can_build_df:
            return hp.build_df()

        if cust_inst.can_join_hp_multiindex_df:
            return cust_inst.join_hp_multiindex_df(hp.df, use_field_names_in_headers)

        if cust_inst.can_build_df:
            return cust_inst.build_df(use_field_names_in_headers)

        return self.build_common_df(
            adc, hp, cust_inst, universe, fields, interval, use_field_names_in_headers
        )

    def build_common_df(
        self,
        adc: "ADCContext",
        hp: "HPContext",
        cust_inst: "CustInstContext",
        universe: "UniverseContainer",
        fields: "FieldsContainer",
        interval: str,
        use_field_names_in_headers: bool,
    ):
        comm_universe = universe.hp
        headers = None
        if hp.raw:
            hp.prepare_to_build(interval)
            headers = hp.headers
            fields = hp.fields

        if adc.raw:
            adc.prepare_to_build(interval)
            comm_universe = adc.universe
            headers = adc.headers

        data = prepare_data(adc.data, hp.data, comm_universe, fields, self.date_name)
        df = self.dfbuilder.build_date_as_index(
            {"data": data, "headers": headers},
            use_field_names_in_headers,
            use_multiindex=bool(cust_inst.raw),
        )

        if cust_inst.raw:
            cust_inst.prepare_to_build(use_field_names_in_headers, df, headers)
            df = cust_inst.df

        df = convert_dtypes(df)

        if len(comm_universe) > 1:
            df.rename(columns={k: v for k, v in enumerate(comm_universe)}, inplace=True)

        if interval is not None and interval not in NON_INTRA_DAY_INTERVALS:
            df.index.names = ["Timestamp"]

        df.sort_index(ascending=True, inplace=True)
        df.ohlc = ohlc.__get__(df, None)

        return df


def get_column_to_idx(columns):
    columns = [
        column.upper() if column.startswith("TR") else column for column in columns
    ]
    column_to_idx = {"Date": 1}
    column_to_idx.update({column: columns.index(column) + 2 for column in columns})
    return column_to_idx


def merge_data(adc_data, hp_data, index):
    adc = adc_data[index] if index < len(adc_data) else []
    hp = hp_data[index] if index < len(hp_data) else []
    return [*adc, *hp]


def add_one_data_item_to_data(
    data_item, data, column_to_idx, columns, instrument, is_one_inst, index
):
    data_item_values = next(iter(data_item.values()))
    for data_item_value in data_item_values:
        data_item = [instrument] if is_one_inst else [index]
        data_item_value_keys = {key.casefold(): key for key in data_item_value.keys()}

        for column in columns:
            column_casefold = column.casefold()
            if column_casefold in data_item_value_keys:
                data_item.insert(
                    column_to_idx[column],
                    data_item_value.get(data_item_value_keys[column_casefold]),
                )

            else:
                data_item.insert(column_to_idx[column], None)

        data.append(data_item)


def add_many_data_item_to_data(
    data_item, data, column_to_idx, columns, instrument, is_one_inst, index, date_name
):
    for one, two in zip_longest(
        *[sorted(item, key=lambda d: d[date_name]) for item in data_item.values()],
        fillvalue=None,
    ):
        data_item = [instrument] if is_one_inst else [index]

        if all([one, two]):
            item = {**one, **two}

        else:
            item = one or two

        key_casefold_to_key_map = {key.casefold(): key for key in item.keys()}
        for column in columns:
            column_casefold = column.casefold()
            if column_casefold in key_casefold_to_key_map.keys():
                data_item.insert(
                    column_to_idx[column],
                    item.get(key_casefold_to_key_map[column_casefold]),
                )

            else:
                data_item.insert(column_to_idx[column], None)

        data.append(data_item)


def prepare_data(
    adc_data: List[list],
    hp_data: List[list],
    instruments: "UniverseContainer",
    columns: List[str],
    date_name: str,
) -> List[list]:

    is_one_inst = len(instruments) == 1
    data = []
    column_to_idx = get_column_to_idx(columns)

    for index, instrument in enumerate(instruments):
        merged_data = merge_data(adc_data, hp_data, index)
        unique_dates = list(dict.fromkeys([item[date_name] for item in merged_data]))
        columns = column_to_idx.keys()

        for date_item in unique_dates:
            group_items = groupby(
                [item for item in merged_data if item[date_name] == date_item],
                lambda item: item["Type"],
            )

            data_item = {}
            for item_type, items in group_items:
                data_item[item_type] = list(items)

            if len(data_item) == 1:
                add_one_data_item_to_data(
                    data_item,
                    data,
                    column_to_idx,
                    columns,
                    instrument,
                    is_one_inst,
                    index,
                )

            else:
                add_many_data_item_to_data(
                    data_item,
                    data,
                    column_to_idx,
                    columns,
                    instrument,
                    is_one_inst,
                    index,
                    date_name,
                )

    return data


def chunks(generator, chunk_size):
    """Yield successive chunks from a generator"""
    chunk = []

    for item in generator:
        if len(chunk) >= chunk_size:
            yield chunk
            chunk = [item]
        else:
            chunk.append(item)

    if chunk:
        yield chunk


def group_universe(universe: List[str]) -> List[tuple]:
    valid_universe = (
        i for i in universe if not (i.startswith("0#.") or i.startswith("Peers("))
    )
    return [(name, len(list(items))) for name, items in groupby(valid_universe)]



import allure
import pytest

from refinitiv.data.content import estimates
from refinitiv.data.errors import RDError
from tests.integration.constants_list import HttpStatusCode, HttpReason
from tests.integration.content.estimates.conftest import (
    invalid_view_actuals_interim_estimates,
    view_actuals_annual_estimates,
    view_actuals_kpi_annual_estimates,
    view_summary_annual_estimates,
    view_summary_historical_non_periodic_estimates,
    view_summary_historical_annual_estimates,
    view_summary_recommendations_estimates,
    view_summary_kpi_annual_estimates,
)
from tests.integration.helpers import (
    get_async_response_from_definitions,
    check_response_status,
    check_non_empty_response_data,
    check_extended_params_were_sent_in_request,
    check_response_dataframe_contains_columns_names,
    check_dataframe_column_date_for_datetime_type,
)


@allure.suite("Content object - Estimates")
@allure.feature("Content object - Estimates")
@allure.severity(allure.severity_level.CRITICAL)
class TestEstimates:
    @allure.title(
        "Create estimates definition object with valid params - synchronously"
    )
    @pytest.mark.caseid("C40050717")
    @pytest.mark.parametrize(
        "estimates_definition,universe,package,header",
        [
            (
                view_actuals_annual_estimates,
                "IBM",
                estimates.Package.PROFESSIONAL,
                "name",
            ),
            (
                view_summary_annual_estimates,
                ["IBM.N", "ORCL.N"],
                estimates.Package.BASIC,
                "title",
            ),
        ],
    )
    @pytest.mark.smoke
    def test_estimates_definition_object_with_valid_params_and_get_data(
        self,
        estimates_definition,
        open_platform_session_with_rdp_creds,
        universe,
        package,
        header,
    ):
        response = estimates_definition(universe=universe, package=package)
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_dataframe_column_date_for_datetime_type(response)

    @allure.title(
        "Create estimates definition object with use_field_names_in_headers True"
    )
    @pytest.mark.caseid("C40050718")
    @pytest.mark.parametrize(
        "universe,expected_column_names",
        [
            (
                ["GOOG.O", "MSFT.O", "FB.O", "AMZN.O", "TWTR.K"],
                [
                    "instrument",
                    "periodenddate",
                    "fperiod",
                    "rfperiod",
                    "TR.AdRevenueActValue",
                ],
            )
        ],
    )
    def test_estimates_definition_object_with_use_field_names_in_headers_true(
        self,
        open_platform_session_with_rdp_creds,
        universe,
        expected_column_names,
    ):
        response = view_actuals_kpi_annual_estimates(
            universe, use_field_names_in_headers=True
        )
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_response_dataframe_contains_columns_names(response, expected_column_names)

    @allure.title(
        "Create estimates definition object with invalid params and get error"
    )
    @pytest.mark.caseid("C40050719")
    @pytest.mark.parametrize(
        "universe,package,expected_error",
        [
            (
                [],
                estimates.Package.PROFESSIONAL,
                "Error code 400 | Validation error: validation failure list:\nuniverse.0 in query should be at least 1 chars long",
            ),
            (
                ["INVALID"],
                estimates.Package.BASIC,
                "Error code 412 | Unable to resolve all requested identifiers. Requested items: ['INVALID']",
            ),
        ],
    )
    def test_estimates_definition_object_with_invalid_params_and_get_error(
        self,
        universe,
        package,
        expected_error,
        open_platform_session_with_rdp_creds,
    ):
        with pytest.raises(RDError) as error:
            view_summary_annual_estimates(universe=universe, package=package)
        assert str(error.value) == expected_error

    @allure.title("Create estimates definition object using closed session")
    @pytest.mark.caseid("C40050720")
    def test_estimates_definition_object_using_closed_session(
        self, open_platform_session_with_rdp_creds
    ):
        session = open_platform_session_with_rdp_creds
        session.close()
        with pytest.raises(ValueError) as error:
            view_actuals_annual_estimates(
                universe="IBM.N", package=estimates.Package.PROFESSIONAL
            )
        assert str(error.value) == "Session is not opened. Can't send any request"

    @allure.title("Create estimates definition object with wrong package")
    @pytest.mark.caseid("C40050721")
    def test_estimates_definition_object_with_wrong_package(
        self, open_platform_session_with_rdp_creds
    ):
        with pytest.raises(AttributeError) as error:
            view_summary_recommendations_estimates(
                universe="IBM.N", package="WRONG_PACKAGE"
            )
        assert (
            str(error.value)
            == "Value 'WRONG_PACKAGE' must be in ['basic', 'standard', 'professional']"
        )

    @allure.title("Create estimates definition object with extended params")
    @pytest.mark.caseid("C40050723")
    @pytest.mark.parametrize(
        "estimate_definition,universe,package,extended_params",
        [
            (
                view_summary_historical_annual_estimates,
                "IBM.N",
                estimates.Package.PROFESSIONAL,
                {"universe": "ORCL.N"},
            ),
            (
                view_summary_historical_non_periodic_estimates,
                "ORCL.N",
                estimates.Package.PROFESSIONAL,
                {"package": "basic"},
            ),
        ],
    )
    def test_estimates_definition_object_with_extended_params(
        self,
        open_platform_session_with_rdp_creds,
        estimate_definition,
        universe,
        package,
        extended_params,
    ):
        response = estimate_definition(universe, package, extended_params)
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_extended_params_were_sent_in_request(response, extended_params)

    @allure.title(
        "Create estimates definition object with use mix valid and invalid data and get partial result"
    )
    @pytest.mark.caseid("C40050722")
    @pytest.mark.parametrize(
        "universe,invalid_universe", [(["ORCL.N", "INVALID"], "INVALID")]
    )
    async def test_estimates_definition_object_with_use_mix_valid_and_invalid_data(
        self, open_platform_session_with_rdp_creds_async, universe, invalid_universe
    ):
        valid_response, invalid_response = await get_async_response_from_definitions(
            view_summary_kpi_annual_estimates(universe=universe),
            invalid_view_actuals_interim_estimates(universe=invalid_universe),
        )
        check_response_status(
            response=valid_response,
            expected_status_code=HttpStatusCode.TWO_HUNDRED,
            expected_http_reason=HttpReason.OK,
            expected_error_code=None,
            expected_error_message="Failed to resolve identifiers ['INVALID']",
        )
        check_non_empty_response_data(valid_response)
        check_response_status(
            response=invalid_response,
            expected_status_code=HttpStatusCode.TWO_HUNDRED,
            expected_http_reason=HttpReason.OK,
            expected_error_code=HttpStatusCode.FOUR_HUNDRED_TWELVE,
            expected_error_message="Unable to resolve all requested identifiers. Requested items: ['INVALID']",
        )



from datetime import datetime, timedelta

import allure
import numpy
import pytest

from refinitiv.data.content.ipa.dates_and_calendars import date_schedule
from refinitiv.data.errors import RDError
from tests.integration.constants_list import HttpStatusCode, HttpReason
from tests.integration.helpers import (
    check_response_status,
    get_async_response_from_definitions,
    check_dataframe_column_date_for_datetime_type,
    check_non_empty_response_data,
    check_response_dataframe_contains_columns_names,
    check_extended_params_were_sent_in_request,
)

first_definition = date_schedule.Definition(
    start_date=timedelta(-180),
    end_date=timedelta(-10),
    calendars=["BAR", "KOR", "JAP"],
    currencies=["USD"],
    frequency=date_schedule.DateScheduleFrequency.WEEKLY,
    day_of_week=date_schedule.DayOfWeek.MONDAY,
)


invalid_definition = date_schedule.Definition(
    start_date="2021.04.24",
    end_date=timedelta(-10),
    calendars=["BAR", "KOR", "JAP"],
    frequency=date_schedule.DateScheduleFrequency.WEEKLY,
)


@allure.suite("Dates And Calendars")
@allure.feature("Content object - Date Schedule")
@allure.severity(allure.severity_level.CRITICAL)
class TestDateSchedule:
    @allure.title("Create a date_schedule definition with valid params")
    @pytest.mark.caseid("C45666963")
    @pytest.mark.parametrize(
        "start_date, end_date, calendars, currencies, count, frequency, day_of_week",
        [
            (
                "2018-12-31",
                "2019-12-31",
                ["UKR", "FRA"],
                ["EUR"],
                None,
                date_schedule.DateScheduleFrequency.WEEKLY,
                date_schedule.DayOfWeek.MONDAY,
            ),
            (
                timedelta(-60),
                timedelta(0),
                ["UKR", "FRA"],
                ["EUR"],
                None,
                "Daily",
                None,
            ),
            (
                datetime(2019, 4, 13),
                None,
                ["UKR", "FRA"],
                ["EUR"],
                20,
                "BiWeekly",
                "Wednesday",
            ),
            ("2019-12-31", datetime.now(), None, ["EUR"], None, "BiWeekly", "Friday"),
        ],
    )
    def test_date_schedule_definition_with_valid_params(
        self,
        open_session,
        start_date,
        end_date,
        calendars,
        currencies,
        count,
        frequency,
        day_of_week,
    ):
        response = date_schedule.Definition(
            start_date=start_date,
            end_date=end_date,
            calendars=calendars,
            currencies=currencies,
            count=count,
            frequency=frequency,
            day_of_week=day_of_week,
        ).get_data()

        assert isinstance(response.data.dates, list)
        assert isinstance(response.data.dates[0], numpy.datetime64)

        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_response_dataframe_contains_columns_names(response, ["dates"])
        check_dataframe_column_date_for_datetime_type(response)

    @allure.title(
        "Create a date_schedule definition with valid and invalid definition asynchronously"
    )
    @pytest.mark.caseid("C46670889")
    async def test_date_schedule_definition_with_valid_and_invalid_def_async(
        self, open_session_async
    ):
        valid_response, invalid_response = await get_async_response_from_definitions(
            first_definition, invalid_definition
        )

        check_non_empty_response_data(valid_response)
        check_response_status(
            response=valid_response,
            expected_status_code=HttpStatusCode.TWO_HUNDRED,
            expected_http_reason=HttpReason.OK,
        )

        check_response_status(
            response=invalid_response,
            expected_status_code=HttpStatusCode.FOUR_HUNDRED,
            expected_http_reason=HttpReason.BAD_REQUEST,
            expected_error_code=HttpStatusCode.FOUR_HUNDRED,
            expected_error_message="Invalid input: 'dayOfWeek' must be specified when 'frequency' is set to 'Weekly' or 'BiWeekly'",
        )

    @allure.title("Create a date_schedule with end_date and count fields and get error")
    @pytest.mark.parametrize(
        "expected_error",
        [
            (
                "Error code 400 | Invalid input: only one of 'endDate' or 'count' must be set."
            )
        ],
    )
    @pytest.mark.caseid("C45667021")
    def test_date_schedule_definition_with_invalid_param(
        self, open_session, expected_error
    ):
        with pytest.raises(RDError) as error:
            date_schedule.Definition(
                start_date="2018-12-31",
                end_date="2019-12-31",
                calendars=["UKR", "FRA"],
                count=5,
                frequency=date_schedule.DateScheduleFrequency.WEEKLY,
                day_of_week=date_schedule.DayOfWeek.MONDAY,
            ).get_data()
        assert str(error.value) == expected_error

    @allure.title("Create date_schedule definition with extended params and get data")
    @pytest.mark.parametrize(
        "extended_param",
        [
            {
                "startDate": "2022-07-04",
                "calendars": ["USA"],
            }
        ],
    )
    @pytest.mark.caseid("C45666968")
    def test_date_schedule_definition_with_extended_params(
        self, open_session, extended_param
    ):
        response = date_schedule.Definition(
            start_date=timedelta(-360),
            end_date="2022-08-31",
            calendars=["UKR", "FRA"],
            currencies=["EUR"],
            extended_params=extended_param,
            frequency=date_schedule.DateScheduleFrequency.WEEKLY,
            day_of_week=date_schedule.DayOfWeek.MONDAY,
        ).get_data()

        check_extended_params_were_sent_in_request(response, extended_param)
        check_non_empty_response_data(response)
        check_response_status(
            response=response,
            expected_status_code=HttpStatusCode.TWO_HUNDRED,
            expected_http_reason=HttpReason.OK,
        )



import asyncio

import pytest

from refinitiv.data import _configure as configure

TIME_OUT = 1


@pytest.mark.asyncio
async def test_rewrite_user_config(write_user_config):
    write_user_config({"prop": "value", "config-change-notifications-enabled": True})

    configure.reload()

    assert "value" == configure.get_config().get("prop")

    write_user_config('{"prop":"new_value"}')

    await asyncio.sleep(TIME_OUT)

    assert "new_value" == configure.get_config().get("prop")


@pytest.mark.asyncio
async def test_rewrite_project_config(write_project_config):
    write_project_config({"prop": "value", "config-change-notifications-enabled": True})

    configure.reload()

    assert "value" == configure.get_config().get("prop")

    write_project_config('{"prop":"new_value"}')

    await asyncio.sleep(TIME_OUT)

    assert "new_value" == configure.get_config().get("prop")


@pytest.mark.asyncio
async def test_update_event(write_project_config):
    fut = asyncio.Future()

    def on_config_updated():
        assert "new_value" == configure.get_config().get("prop")
        if not fut.done():
            fut.set_result(1)

    write_project_config('{"prop":"value"}')
    configure.reload()
    configure.get_config().on("update", on_config_updated)
    write_project_config('{"prop":"new_value"}')

    await asyncio.sleep(TIME_OUT)

    await fut


@pytest.mark.asyncio
async def test_update_event(write_project_config):
    def on_config_updated():
        assert False

    write_project_config({"prop": "value", "config-change-notifications-enabled": True})
    configure.reload()
    configure.get_config().on("update", on_config_updated)
    configure.get_config().remove_listener("update", on_config_updated)
    write_project_config('{"prop":"new_value"}')

    await asyncio.sleep(TIME_OUT)

    assert "new_value" == configure.get_config().get("prop")



# coding: utf8

__all__ = ["NumericalMethod"]

from .._object_definition import ObjectDefinition
from .._enums import Method
from .._models import AmericanMonteCarloParameters, PdeParameters


class NumericalMethod(ObjectDefinition):
    def __init__(
        self, american_monte_carlo_parameters=None, method=None, pde_parameters=None
    ):
        super().__init__()
        self.american_monte_carlo_parameters = american_monte_carlo_parameters
        self.method = method
        self.pde_parameters = pde_parameters

    @property
    def american_monte_carlo_parameters(self):
        """
        :return: object AmericanMonteCarloParameters
        """
        return self._get_object_parameter(
            AmericanMonteCarloParameters, "americanMonteCarloParameters"
        )

    @american_monte_carlo_parameters.setter
    def american_monte_carlo_parameters(self, value):
        self._set_object_parameter(
            AmericanMonteCarloParameters, "americanMonteCarloParameters", value
        )

    @property
    def method(self):
        """
        :return: enum Method
        """
        return self._get_enum_parameter(Method, "method")

    @method.setter
    def method(self, value):
        self._set_enum_parameter(Method, "method", value)

    @property
    def pde_parameters(self):
        """
        :return: object PdeParameters
        """
        return self._get_object_parameter(PdeParameters, "pdeParameters")

    @pde_parameters.setter
    def pde_parameters(self, value):
        self._set_object_parameter(PdeParameters, "pdeParameters", value)



# coding: utf8


from typing import TYPE_CHECKING, Optional

from .omm_stream import OMMStream
from ..._tools import create_repr, fields_arg_parser, try_copy_to_list

if TYPE_CHECKING:
    from ..._types import ExtendedParams, OptStrStrs
    from ..._core.session import Session


class Definition:
    """
    This class to subscribe to streaming items of any Domain Model
    (e.g. MarkePrice, MarketByPrice, ...)
    exposed by the underlying of the Refinitiv Data

    Parameters
    ----------
    name : str, optional
        Name of the streaming instrument
    api: str, optional
        Specifies the data source. It can be updated/added using config file
    service : str, optional
        Offers the ability to use specific service to manage the
        real-time streaming data
    fields : str or list, optional
        Specifies the specific fields delivered when messages arrive
    domain : str, optional
        Defines the domain for the specific data of interest
        Default : "MarketPrice"
    extended_params : dict, optional
        Other parameters can be provided if necessary

    Examples
    --------
    >>> from refinitiv.data.delivery import omm_stream
    >>> definition = omm_stream.Definition("EUR")
    """

    def __init__(
        self,
        name: str,
        api: Optional[str] = None,
        service: Optional[str] = None,
        fields: "OptStrStrs" = None,
        domain: str = "MarketPrice",
        extended_params: "ExtendedParams" = None,
    ) -> None:
        self._name = name
        self._api = api
        self._domain = domain
        self._service = service
        fields = try_copy_to_list(fields)
        self._fields = fields and fields_arg_parser.get_list(fields)
        self._extended_params = extended_params

    def __repr__(self):
        content = f"{{name='{self._name}'}}"
        return create_repr(self, middle_path="omm_stream", content=content)

    def get_stream(self, session: "Session" = None) -> OMMStream:
        """
        Initialization subscribe to streaming items

        Parameters
        ----------
        session: Session, optional
            The Session defines the source where you want to retrieve your data

        Returns
        -------
        current instance

        Examples
        --------
        >>> from refinitiv.data.delivery import omm_stream
        >>> definition = omm_stream.Definition("EUR")
        >>> stream = definition.get_stream()
        """
        stream = OMMStream(
            session=session,
            name=self._name,
            api=self._api,
            service=self._service,
            fields=self._fields,
            domain=self._domain,
            extended_params=self._extended_params,
        )
        return stream



from enum import Enum, unique


@unique
class IndustryGroup(Enum):
    AEROSPACE_AND_DEFENSE = "AerospaceAndDefense"
    AUTOMOBILES_AND_AUTO_PARTS = "AutomobilesAndAutoParts"
    BANKING_AND_INVESTMENT_SERVICES = "BankingAndInvestmentServices"
    BANKING_SERVICES = "BankingServices"
    BEVERAGES = "Beverages"
    BIOTECHNOLOGY_AND_MEDICAL_RESEARCH = "BiotechnologyAndMedicalResearch"
    CHEMICALS = "Chemicals"
    COAL = "Coal"
    COLLECTIVE_INVESTMENTS = "CollectiveInvestments"
    COMMUNICATIONS_AND_NETWORKING = "CommunicationsAndNetworking"
    COMPUTERS_PHONES_AND_HOUSEHOLD_ELECTRONICS = (
        "ComputersPhonesAndHouseholdElectronics"
    )
    CONSTRUCTION_AND_ENGINEERING = "ConstructionAndEngineering"
    CONSTRUCTION_MATERIALS = "ConstructionMaterials"
    CONSUMER_GOODS_CONGLOMERATES = "ConsumerGoodsConglomerates"
    CONTAINERS_AND_PACKAGING = "ContainersAndPackaging"
    DIVERSIFIED_INDUSTRIAL_GOODS_WHOLESALE = "DiversifiedIndustrialGoodsWholesale"
    DIVERSIFIED_RETAIL = "DiversifiedRetail"
    ELECTRIC_UTILITIES_AND_IP_PS = "ElectricUtilitiesAndIPPs"
    ELECTRONIC_EQUIPMENT_AND_PARTS = "ElectronicEquipmentAndParts"
    FINANCIAL_TECHNOLOGY_AND_INFRASTRUCTURE = "FinancialTechnologyAndInfrastructure"
    FOOD_AND_DRUG_RETAILING = "FoodAndDrugRetailing"
    FOOD_AND_TOBACCO = "FoodAndTobacco"
    FREIGHT_AND_LOGISTICS_SERVICES = "FreightAndLogisticsServices"
    GOVERNMENT_ACTIVITY = "GovernmentActivity"
    HEALTHCARE_EQUIPMENT_AND_SUPPLIES = "HealthcareEquipmentAndSupplies"
    HEALTHCARE_PROVIDERS_AND_SERVICES = "HealthcareProvidersAndServices"
    HOMEBUILDING_AND_CONSTRUCTION_SUPPLIES = "HomebuildingAndConstructionSupplies"
    HOTELS_AND_ENTERTAINMENT_SERVICES = "HotelsAndEntertainmentServices"
    HOUSEHOLD_GOODS = "HouseholdGoods"
    INSTITUTIONS_ASSOCIATIONS_AND_ORGANIZATIONS = (
        "InstitutionsAssociationsAndOrganizations"
    )
    INSURANCE = "Insurance"
    INTEGRATED_HARDWARE_AND_SOFTWARE = "IntegratedHardwareAndSoftware"
    INVESTMENT_BANKING_AND_INVESTMENT_SERVICES = (
        "InvestmentBankingAndInvestmentServices"
    )
    INVESTMENT_HOLDING_COMPANIES = "InvestmentHoldingCompanies"
    LEISURE_PRODUCTS = "LeisureProducts"
    MACHINERY_TOOLS_HEAVY_VEHICLES_TRAINS_AND_SHIPS = (
        "MachineryToolsHeavyVehiclesTrainsAndShips"
    )
    MEDIA_AND_PUBLISHING = "MediaAndPublishing"
    METALS_AND_MINING = "MetalsAndMining"
    MISCELLANEOUS_EDUCATIONAL_SERVICE = "MiscellaneousEducationalService"
    MULTILINE_UTILITIES = "MultilineUtilities"
    NATURAL_GAS_UTILITIES = "NaturalGasUtilities"
    OFFICE_EQUIPMENT = "OfficeEquipment"
    OIL_AND_GAS = "OilAndGas"
    OIL_AND_GAS_RELATED_EQUIPMENT_AND_SERVICES = "OilAndGasRelatedEquipmentAndServices"
    PAPER_AND_FOREST_PRODUCTS = "PaperAndForestProducts"
    PASSENGER_TRANSPORTATION_SERVICES = "PassengerTransportationServices"
    PERSONAL_AND_HOUSEHOLD_PRODUCTS_AND_SERVICES = (
        "PersonalAndHouseholdProductsAndServices"
    )
    PHARMACEUTICALS = "Pharmaceuticals"
    PROFESSIONAL_AND_BUSINESS_EDUCATION = "ProfessionalAndBusinessEducation"
    PROFESSIONAL_AND_COMMERCIAL_SERVICES = "ProfessionalAndCommercialServices"
    PROVIDERS = "Providers"
    REAL_ESTATE_OPERATIONS = "RealEstateOperations"
    RENEWABLE_ENERGY = "RenewableEnergy"
    RESIDENTIAL_AND_COMMERCIAL_REI_TS = "ResidentialAndCommercialREITs"
    SCHOOL_COLLEGE_AND_UNIVERSITY = "SchoolCollegeAndUniversity"
    SEMICONDUCTORS_AND_SEMICONDUCTOR_EQUIPMENT = (
        "SemiconductorsAndSemiconductorEquipment"
    )
    SOFTWARE_AND_IT_SERVICES = "SoftwareAndITServices"
    SPECIALTY_RETAILERS = "SpecialtyRetailers"
    TELECOMMUNICATIONS_SERVICES = "TelecommunicationsServices"
    TEXTILES_AND_APPAREL = "TextilesAndApparel"
    TRANSPORT_INFRASTRUCTURE = "TransportInfrastructure"
    URANIUM = "Uranium"
    WATER_AND_RELATED_UTILITIES = "WaterAndRelatedUtilities"



from types import SimpleNamespace
from typing import List, Union

from ._content_data import Data
from ._df_builder import build_empty_df
from ..delivery._data._parsed_data import ParsedData
from ..delivery._data._response import Response


def historical_join_responses(
    responses: List[Response], data: Union["Data", dict]
) -> Response:
    errors = []
    http_statuses = []
    http_headers = []
    http_responses = []
    request_messages = []

    for response in responses:
        http_statuses.append(response.http_status)
        http_headers.append(response.http_headers)
        request_messages.append(response.request_message)
        http_responses.append(response.http_response)

        if response.errors:
            errors += response.errors

    raw_response = SimpleNamespace()
    raw_response.request = request_messages
    raw_response.headers = http_headers
    data_factory = SimpleNamespace()
    response = Response(
        any(r.is_success for r in responses),
        ParsedData(http_statuses, raw_response),
        data_factory,
    )
    data_factory.create_data = (
        lambda *args, **kwargs: data
        if isinstance(data, Data)
        else Data(data, build_empty_df, owner_=response)
    )
    response.errors += errors
    response.http_response = http_responses

    return response



import mock
from unittest.mock import MagicMock

from refinitiv.data.discovery import convert_symbols, SymbolTypes


def test_convert_symbols():
    mocked_response = MagicMock()
    with mock.patch(
        "refinitiv.data.content.symbol_conversion.Definition.get_data",
        new=mocked_response,
    ):
        convert_symbols(
            symbols=["US5949181045", "US02079K1079"],
            from_symbol_type=SymbolTypes.ISIN,
            to_symbol_types="RIC",
        )
        assert mocked_response.call_count == 1



# coding: utf8

from typing import Optional

from .._base import UnderlyingDefinition
from ..._instrument_definition import InstrumentDefinition
from .._enums import (
    BuySell,
    CallPut,
    ExerciseStyle,
    UnderlyingType,
)
from ._eti_barrier_definition import EtiBarrierDefinition
from ._eti_binary_definition import EtiBinaryDefinition
from ._eti_cbbc_definition import EtiCbbcDefinition
from ._eti_double_barriers_definition import EtiDoubleBarriersDefinition
from ._eti_fixing_info import EtiFixingInfo
from ._eti_underlying_definition import EtiUnderlyingDefinition


class EtiDefinition(InstrumentDefinition):
    """
    Parameters
    ----------
    instrument_tag : str, optional
        User defined string to identify the instrument.It can be used to link output
        results to the instrument definition. Only alphabetic, numeric and '- _.#=@'
        characters are supported. Optional.
    instrument_code : str, optional
        An option RIC that is used to retrieve the description of the
        EtiOptionDefinition contract. Optional.If null, the instrument_code of
        underlying_definition must be provided.
    end_date : str, optional
        Expiry date of the option
    asian_definition : EtiOptionFixingInfo, optional
        Fixing details for asian options
    barrier_definition : EtiOptionBarrierDefinition, optional
        Details for barrier option.
    binary_definition : EtiOptionBinaryDefinition, optional
        Details for binary option.
    buy_sell : BuySell, optional
        The side of the deal. Possible values:
        - Buy
        - Sell
    call_put : CallPut, optional
        Tells if the option is a call or a put. Possible values:
        - Call
        - Put
    cbbc_definition : EtiOptionCbbcDefinition, optional
        Details for CBBC (Call Bear/Bull Contract) option.
    double_barriers_definition : EtiOptionDoubleBarriersDefinition, optional
        Details for double barriers option.
    exercise_style : ExerciseStyle, optional
        EURO or AMER
    underlying_definition : EtiUnderlyingDefinition, optional
        Details of the underlying. Can be used to override some data of the underlying.
    underlying_type : UnderlyingType, optional
        Underlying type of the option. Possible values:
        - Eti
        - Fx
    deal_contract : int, optional
        deal_contract. It is the number of contracts bought or sold in the deal.
    end_date_time : str, optional
        Expiry date time of the option
    lot_size : float, optional
        The lot size. It is the number of options bought or sold in one transaction.
    offset : int, optional
        offset. The offset in minutes between the time UTC and the time of the exchange
        where the contract is traded.
    strike : float, optional
        strike of the option
    """

    def __init__(
        self,
        instrument_tag: Optional[str] = None,
        instrument_code: Optional[str] = None,
        start_date: Optional[str] = None,
        end_date: Optional[str] = None,
        asian_definition: Optional[EtiFixingInfo] = None,
        barrier_definition: Optional[EtiBarrierDefinition] = None,
        binary_definition: Optional[EtiBinaryDefinition] = None,
        buy_sell: Optional[BuySell] = None,
        call_put: Optional[CallPut] = None,
        cbbc_definition: Optional[EtiCbbcDefinition] = None,
        double_barriers_definition: Optional[EtiDoubleBarriersDefinition] = None,
        exercise_style: Optional[ExerciseStyle] = None,
        underlying_definition: Optional[EtiUnderlyingDefinition] = None,
        underlying_type: Optional[UnderlyingType] = None,
        deal_contract: Optional[int] = None,
        end_date_time: Optional[str] = None,
        lot_size: Optional[float] = None,
        strike: Optional[float] = None,
        time_zone_offset: Optional[int] = None,
        **kwargs,
    ) -> None:
        super().__init__(instrument_tag, **kwargs)
        self.instrument_tag = instrument_tag
        self.instrument_code = instrument_code
        self.start_date = start_date
        self.end_date = end_date
        self.asian_definition = asian_definition
        self.barrier_definition = barrier_definition
        self.binary_definition = binary_definition
        self.buy_sell = buy_sell
        self.call_put = call_put
        self.cbbc_definition = cbbc_definition
        self.double_barriers_definition = double_barriers_definition
        self.exercise_style = exercise_style
        self.underlying_definition = underlying_definition
        self.underlying_type = underlying_type
        self.deal_contract = deal_contract
        self.end_date_time = end_date_time
        self.lot_size = lot_size
        self.strike = strike
        self.time_zone_offset = time_zone_offset

    @property
    def asian_definition(self):
        """
        :return: object EtiOptionFixingInfo
        """
        return self._get_object_parameter(EtiFixingInfo, "asianDefinition")

    @asian_definition.setter
    def asian_definition(self, value):
        self._set_object_parameter(EtiFixingInfo, "asianDefinition", value)

    @property
    def barrier_definition(self):
        """
        :return: object EtiOptionBarrierDefinition
        """
        return self._get_object_parameter(EtiBarrierDefinition, "barrierDefinition")

    @barrier_definition.setter
    def barrier_definition(self, value):
        self._set_object_parameter(EtiBarrierDefinition, "barrierDefinition", value)

    @property
    def binary_definition(self):
        """
        :return: object EtiOptionBinaryDefinition
        """
        return self._get_object_parameter(EtiBinaryDefinition, "binaryDefinition")

    @binary_definition.setter
    def binary_definition(self, value):
        self._set_object_parameter(EtiBinaryDefinition, "binaryDefinition", value)

    @property
    def buy_sell(self):
        """
        The indicator of the deal side. the possible values are:
        - buy: buying the option,
        - sell: selling/writing the option. the output amounts calculated with taking
          buysell into consideration are returned with a reversed sign when the value
          'sell' is used. optional. the default value is 'buy'.
        :return: enum BuySell
        """
        return self._get_enum_parameter(BuySell, "buySell")

    @buy_sell.setter
    def buy_sell(self, value):
        self._set_enum_parameter(BuySell, "buySell", value)

    @property
    def call_put(self):
        """
        The indicator if the option is a call or a put. the possible values are:
        - call: the right to buy the underlying asset,
        - put: the right to sell the underlying asset. optional. if instrumentcode of
          listed eti option is defined, the value comes from the instrument reference
          data.the default value is 'call' for otc eti options and fx options.
        :return: enum CallPut
        """
        return self._get_enum_parameter(CallPut, "callPut")

    @call_put.setter
    def call_put(self, value):
        self._set_enum_parameter(CallPut, "callPut", value)

    @property
    def cbbc_definition(self):
        """
        :return: object EtiOptionCbbcDefinition
        """
        return self._get_object_parameter(EtiCbbcDefinition, "cbbcDefinition")

    @cbbc_definition.setter
    def cbbc_definition(self, value):
        self._set_object_parameter(EtiCbbcDefinition, "cbbcDefinition", value)

    @property
    def double_barriers_definition(self):
        """
        :return: object EtiOptionDoubleBarriersDefinition
        """
        return self._get_object_parameter(
            EtiDoubleBarriersDefinition, "doubleBarriersDefinition"
        )

    @double_barriers_definition.setter
    def double_barriers_definition(self, value):
        self._set_object_parameter(
            EtiDoubleBarriersDefinition, "doubleBarriersDefinition", value
        )

    @property
    def exercise_style(self):
        """
        The option style based on its exercise restrictions. the possible values are:
        - amer: the owner has the right to exercise on any date before the option
          expires,
        - euro: the owner has the right to exercise only on enddate,
        - berm: the owner has the right to exercise on any of several specified dates
          before the option expires. all exercise styles may not apply to certain option
          types. optional. if instrumentcode of listed eti option is defined, the value
          comes from the instrument reference data. the default value is 'euro' for otc
          eti options and fx options.
        :return: enum ExerciseStyle
        """
        return self._get_enum_parameter(ExerciseStyle, "exerciseStyle")

    @exercise_style.setter
    def exercise_style(self, value):
        self._set_enum_parameter(ExerciseStyle, "exerciseStyle", value)

    @property
    def underlying_definition(self):
        """
        :return: object EtiUnderlyingDefinition
        """
        return self._get_object_parameter(UnderlyingDefinition, "underlyingDefinition")

    @underlying_definition.setter
    def underlying_definition(self, value):
        self._set_object_parameter(UnderlyingDefinition, "underlyingDefinition", value)

    @property
    def underlying_type(self):
        """
        The type of the option based on the underlying asset. the possible values are:
        - eti: eti(exchanged traded instruments) options,
        - fx: fx options. mandatory. no default value applies.
        :return: enum UnderlyingType
        """
        return self._get_enum_parameter(UnderlyingType, "underlyingType")

    @underlying_type.setter
    def underlying_type(self, value):
        self._set_enum_parameter(UnderlyingType, "underlyingType", value)

    @property
    def deal_contract(self):
        """
        The number of contracts bought or sold in the deal. optional.the default value
        is '1'.
        :return: int
        """
        return self._get_parameter("dealContract")

    @deal_contract.setter
    def deal_contract(self, value):
        self._set_parameter("dealContract", value)

    @property
    def end_date(self):
        """
        The maturity or expiry date of the instrument. the value is expressed in iso
        8601 format: yyyy-mm-ddt[hh]:[mm]:[ss]z (e.g., '2021-01-01t00:00:00z').
        optional. mandatory for otc eti options and fx options(if tenor is not defined).
        if instrumentcode of listed eti option is defined, the value comes from the
        instrument reference data.
        :return: str
        """
        return self._get_parameter("endDate")

    @end_date.setter
    def end_date(self, value):
        self._set_parameter("endDate", value)

    @property
    def end_date_time(self):
        """
        The expiry date and time of the instrument at the exchange where it is traded.
        the value is expressed in iso 8601 format: yyyy-mm-ddt[hh]:[mm]:[ss]z (e.g.,
        '2021-01-01t00:00:00z'). optional. no default value applies.
        :return: str
        """
        return self._get_parameter("endDateTime")

    @end_date_time.setter
    def end_date_time(self, value):
        self._set_parameter("endDateTime", value)

    @property
    def instrument_code(self):
        """
        The code (an option ric) used to define the instrument. optional. mandatory for
        listed eti options. for otc eti options instrumentcode of the underlying asset
        must be provided. no default value applies.
        :return: str
        """
        return self._get_parameter("instrumentCode")

    @instrument_code.setter
    def instrument_code(self, value):
        self._set_parameter("instrumentCode", value)

    @property
    def instrument_tag(self):
        """
        User defined string to identify the instrument.it can be used to link output
        results to the instrument definition. only alphabetic, numeric and '- _.#=@'
        characters are supported. optional.
        :return: str
        """
        return self._get_parameter("instrumentTag")

    @instrument_tag.setter
    def instrument_tag(self, value):
        self._set_parameter("instrumentTag", value)

    @property
    def lot_size(self):
        """
        The number of the underlying asset unit on which the option is written. it can
        be overriden only for commodity options. optional. if instrumentcode of listed
        eti option is defined the value comes from the instrument reference data. the
        default value is '1' for otc eti options.
        :return: float
        """
        return self._get_parameter("lotSize")

    @lot_size.setter
    def lot_size(self, value):
        self._set_parameter("lotSize", value)

    @property
    def start_date(self):
        """
        Start date of the option
        :return: str
        """
        return self._get_parameter("startDate")

    @start_date.setter
    def start_date(self, value):
        self._set_parameter("startDate", value)

    @property
    def strike(self):
        """
        The set price at which the owner of the option can buy or sell the underlying
        asset. the value is expressed according to the market convention linked to the
        underlying asset. optional. mandatory for otc eti options and fx options. if
        instrumentcode of listed eti option is defined, the value comes from the
        instrument reference data.
        :return: float
        """
        return self._get_parameter("strike")

    @strike.setter
    def strike(self, value):
        self._set_parameter("strike", value)

    @property
    def time_zone_offset(self):
        """
        The offset in minutes between utc and the time of the exchange where the
        contract is traded. optional. no default value applies.
        :return: int
        """
        return self._get_parameter("timeZoneOffset")

    @time_zone_offset.setter
    def time_zone_offset(self, value):
        self._set_parameter("timeZoneOffset", value)



import os
from unittest.mock import MagicMock, Mock, patch

import pytest
from httpx import Client

from refinitiv.data import OpenState
from refinitiv.data._core.session import DesktopSession
from refinitiv.data.delivery._data._request import Request
from refinitiv.data.session import desktop


# --------------------------------------------------------------------------------------
# self._base_url
# --------------------------------------------------------------------------------------


def test__base_url_property():
    # given
    expected_value = "http://localhost:9000"
    definition = desktop.Definition(app_key="app_key")
    session = definition.get_session()

    # when
    testing_value = session._get_base_url()

    # then
    assert testing_value == expected_value


# --------------------------------------------------------------------------------------
# self.set_port_number()
# --------------------------------------------------------------------------------------


def test__set_port_number():
    # given
    expected_value = "http://localhost:9000"
    definition = desktop.Definition(app_key="app_key")
    session = definition.get_session()

    session.set_port_number(9600)

    # when
    testing_value = session._get_base_url()

    # then
    assert testing_value == "http://localhost:9600"


# --------------------------------------------------------------------------------------
# self._dp_proxy_base_url
# --------------------------------------------------------------------------------------
def test__dp_proxy_base_url_property():
    # given
    expected_value = "https://dp_proxy_base_url:9000"
    os.environ["DP_PROXY_BASE_URL"] = expected_value
    session = desktop.Definition(app_key="app_key").get_session()

    # when
    testing_value = session._get_base_url()

    # then
    assert testing_value == expected_value

    os.environ.pop("DP_PROXY_BASE_URL")


# --------------------------------------------------------------------------------------
# self._rdp_url
# --------------------------------------------------------------------------------------


def test__rdp_url_property():
    # given
    expected_value = "http://localhost:9000/api/rdp"
    definition = desktop.Definition(app_key="app_key")
    session = definition.get_session()

    # when
    testing_value = session._get_rdp_url_root()

    # then
    assert testing_value == expected_value


# --------------------------------------------------------------------------------------
# self.get_port_number_from_range
# --------------------------------------------------------------------------------------


@pytest.mark.asyncio
async def test__get_port_number_from_range_successfully():
    # given
    expected_port = "36035"

    definition = desktop.Definition(app_key="app_key")
    session = definition.get_session()

    session._connection.debug = Mock()
    session._connection.check_proxy = Mock(return_value=True)

    # when
    testing_value = session._connection.get_port_number_from_range(
        ports=(expected_port,), url="http://localhost:1245"
    )

    # then
    assert testing_value == expected_port
    session._connection.debug.assert_called_with(
        f"Default proxy port {expected_port} was successfully checked"
    )


@pytest.mark.asyncio
async def test__get_port_number_from_range_fail():
    # given
    expected_port = "36035"

    definition = desktop.Definition(app_key="app_key")
    session = definition.get_session()

    session._connection.debug = Mock()
    session._connection.check_proxy = Mock(return_value=False)

    # when
    testing_value = session._connection.get_port_number_from_range(
        ports=(expected_port,), url="http://localhost:1245"
    )

    # then
    assert testing_value is None
    session._connection.debug.assert_called_with(
        f"Default proxy port #{expected_port} failed"
    )


def test_desktop_session_opens_and_closes_with_context_manager():
    def open_mock():
        return True

    # given
    session = DesktopSession(app_key="app_key")
    session._connection.open = open_mock

    # when
    with session:
        assert session.open_state is OpenState.Opened

    # then
    assert session.open_state is OpenState.Closed


@pytest.mark.asyncio
async def test_desktop_session_opens_and_closes_with_async_context_manager():
    def open_mock():
        return True

    # given
    session = DesktopSession(app_key="app_key")
    session._connection.open = open_mock

    # when
    async with session:
        assert session.open_state is OpenState.Opened

    # then
    assert session.open_state is OpenState.Closed



import allure
import pytest

import refinitiv.data.content.ipa.financial_contracts as rdf
from tests.integration.constants_list import HttpStatusCode, HttpReason
from tests.integration.content.ipa.contracts.conftest import (
    financial_contracts_definition_without_fields,
    financial_contracts_definition_with_fields,
    check_http_status_is_success_and_df_value_not_empty,
    check_request_message_contains_expected_fields,
    get_valid_and_invalid_financial_contracts_definition,
)
from tests.integration.helpers import (
    get_async_response_from_definition,
    check_response_dataframe_contains_columns_names,
    check_response_status,
    check_dataframe_column_date_for_datetime_type,
)

COLUMN_NAMES = {
    "argnames": "expected_column_names",
    "argvalues": [
        (
            [
                "InstrumentCode",
                "MarketDataDate",
                "YieldPercent",
                "GovernmentSpreadBp",
                "GovCountrySpreadBp",
                "RatingSpreadBp",
                "SectorRatingSpreadBp",
                "EdsfSpreadBp",
                "IssuerSpreadBp",
                "ErrorMessage",
            ]
        )
    ],
}


@allure.suite("IPA FinancialContracts")
@allure.feature("IPA FinancialContracts")
@allure.severity(allure.severity_level.CRITICAL)
class TestFinancialContracts:
    @allure.title("Create a list of definitions with params and get data")
    @pytest.mark.parametrize(**COLUMN_NAMES)
    @pytest.mark.caseid("38447246")
    def test_user_created_definitions_list(self, open_session, expected_column_names):
        response = financial_contracts_definition_without_fields().get_data()
        check_http_status_is_success_and_df_value_not_empty(response)
        check_response_dataframe_contains_columns_names(response, expected_column_names)
        check_dataframe_column_date_for_datetime_type(response)

    @allure.title(
        "Create a list of definitions with expected fields in dataframe columns"
    )
    @pytest.mark.caseid("38447487")
    @pytest.mark.parametrize(**COLUMN_NAMES)
    def test_user_created_definitions_list_with_custom_fields_1(
        self, open_session, expected_column_names
    ):
        response = financial_contracts_definition_with_fields().get_data()
        check_request_message_contains_expected_fields(response, expected_column_names)
        check_response_dataframe_contains_columns_names(response, expected_column_names)

    @allure.title("Create a list of definitions request asynchronously")
    @pytest.mark.caseid("38447267")
    async def test_user_created_definitions_list_asynchronously(self, open_session_async):
        response = await get_async_response_from_definition(
            financial_contracts_definition_without_fields()
        )
        check_http_status_is_success_and_df_value_not_empty(response)

    @allure.title(
        "Create a list of definitions request asynchronously with mix of valid and invalid parameters"
    )
    @pytest.mark.parametrize(
        "error_message",
        [
            "Analysis error : an error occurred. An unexpected error occurs during analysis step"
        ],
    )
    @pytest.mark.caseid("38447267")
    async def test_user_get_data_async_with_mix_of_valid_and_invalid_params(
        self, error_message, open_session_async
    ):
        definition = get_valid_and_invalid_financial_contracts_definition()
        response = await get_async_response_from_definition(definition)
        check_http_status_is_success_and_df_value_not_empty(response)

        check_response_status(
            response=response,
            expected_status_code=HttpStatusCode.TWO_HUNDRED,
            expected_http_reason=HttpReason.OK,
            expected_error_message=error_message,
        )

    @allure.title(
        "Create a definition list object without required parameter"
    )
    @pytest.mark.caseid("38447268")
    def test_definition_with_invalid_value_type(self, open_session):
        with pytest.raises(TypeError) as error:
            rdf.Definitions().get_data()
        assert str(error.value) == "__init__() missing 1 required positional argument: 'universe'"



__all__ = (
    "fx_swp_to_swp",
    "FxSpotQuote",
    "FxSwapPoints",
    "TenorBidAsk",
    "TenorTypes",
)

from ._fx_swp_to_swp import fx_swp_to_swp
from ._fx_spot_quote import FxSpotQuote
from ._fx_swap_points import FxSwapPoints
from ._tenor_bid_ask import TenorBidAsk
from ._tenor_types import TenorTypes



import json

import allure
import pytest

import refinitiv.data.content.ipa.financial_contracts as rdf
from refinitiv.data.errors import RDError
from tests.integration.content.ipa.contracts.conftest import (
    check_http_status_is_success_and_df_value_not_empty,
    check_stream_state_and_df_from_stream,
)
from tests.integration.content.ipa.contracts.cross.conftest import cross_universe
from tests.integration.helpers import (
    get_async_response_from_definition,
    check_dataframe_column_date_for_datetime_type,
)

fields = [
    "ValuationDate",
    "InstrumentDescription",
    "EndDate",
    "FxSwapsCcy1Ccy2",
    "MarketValueInReportCcy",
    "DeltaAmountInReportCcy",
    "RhoContraCcyAmountInReportCcy",
    "RhoDealCcyAmountInReportCcy",
    "MarketDataDate",
    "ErrorCode",
    "ErrorMessage",
]


@allure.suite("Content object - Cross")
@allure.feature("Content object - Cross")
@allure.severity(allure.severity_level.CRITICAL)
class TestCross:
    @allure.title(
        "Calculate Non-Deliverable Forward Outright USD/INR and Swap Points for a Broken Date"
    )
    @pytest.mark.caseid("36597349")
    @pytest.mark.smoke
    def test_fx_no_deliverable_forward_usd_inr(self, open_session):
        response = rdf.cross.Definition(
            fx_cross_type=rdf.cross.FxCrossType.FX_NON_DELIVERABLE_FORWARD,
            fx_cross_code="USDINR",
            legs=[
                rdf.cross.LegDefinition(
                    deal_amount=1000000,
                    contra_amount=65762500,
                    deal_ccy_buy_sell=rdf.cross.BuySell.BUY,
                    tenor="4Y",
                )
            ],
            pricing_parameters=rdf.cross.PricingParameters(
                valuation_date="2017-11-15T00:00:00Z"
            ),
            fields=fields,
        ).get_data()

        check_http_status_is_success_and_df_value_not_empty(response)
        check_dataframe_column_date_for_datetime_type(response)

    @allure.title("Calculate a FX Forward Outright USD/EUR for a Broken Date")
    @pytest.mark.caseid("36597350")
    def test_fx_forward_usd_eur(self, open_session):
        response = rdf.cross.Definition(
            instrument_tag="00102700008910C",
            fx_cross_type=rdf.cross.FxCrossType.FX_FORWARD,
            fx_cross_code="USDEUR",
            legs=[rdf.cross.LegDefinition(end_date="2015-04-09T00:00:00Z")],
            pricing_parameters=rdf.cross.PricingParameters(
                valuation_date="2015-02-02T00:00:00Z",
                price_side=rdf.cross.PriceSide.MID,
            ),
            fields=fields,
        ).get_data()

        check_http_status_is_success_and_df_value_not_empty(response)

    @allure.title("Calculate a FX Swap CHF/JPY Points for a Broken Date")
    @pytest.mark.caseid("36597356")
    def test_fx_swap_chf_jpy(self, open_session):
        response = rdf.cross.Definition(
            instrument_tag="1Y-CHFJPY",
            fx_cross_type=rdf.cross.FxCrossType.FX_SWAP,
            fx_cross_code="CHFJPY",
            legs=[
                rdf.cross.LegDefinition(
                    deal_ccy_buy_sell=rdf.cross.BuySell.BUY,
                    fx_leg_type=rdf.cross.FxLegType.SWAP_NEAR,
                    deal_amount=1000000,
                    contra_amount=897008.3,
                    tenor="1M",
                ),
                rdf.cross.LegDefinition(
                    deal_ccy_buy_sell=rdf.cross.BuySell.SELL,
                    fx_leg_type=rdf.cross.FxLegType.SWAP_FAR,
                    deal_amount=1000000,
                    contra_amount=900000,
                    tenor="1Y",
                ),
            ],
            pricing_parameters=rdf.cross.PricingParameters(
                valuation_date="2018-02-17T00:00:00Z",
                price_side=rdf.cross.PriceSide.ASK,
            ),
            fields=fields,
        ).get_data()

        check_http_status_is_success_and_df_value_not_empty(response)

    @allure.title("Get a cross analytics FX Forward USD/EUR")
    @pytest.mark.caseid("36597358")
    def test_get_cross_analytics_fx_forward(self, open_session):
        response = rdf.cross.Definition(
            instrument_tag="00102700008910C",
            fx_cross_type=rdf.cross.FxCrossType.FX_FORWARD,
            fx_cross_code="USDEUR",
            traded_cross_rate=123,
            traded_swap_points=123,
            reference_spot_rate=123,
            ndf_fixing_settlement_ccy="",
            legs=[rdf.cross.LegDefinition(end_date="2015-04-09T00:00:00Z")],
            pricing_parameters=rdf.cross.PricingParameters(
                valuation_date="2015-02-02T00:00:00Z",
                price_side=rdf.cross.PriceSide.MID,
            ),
            fields=fields,
        ).get_data()

        check_http_status_is_success_and_df_value_not_empty(response)

    @allure.title("Get cross analytics Non-Deliverable Fx Forward Outright USD/INR")
    @pytest.mark.caseid("36597373")
    def test_get_cross_analytics_fx_non_deliverable_forward(self, open_session):
        response = rdf.cross.Definition(
            instrument_tag="1Y-USDINR",
            fx_cross_type=rdf.cross.FxCrossType.FX_NON_DELIVERABLE_FORWARD,
            fx_cross_code="USDINR",
            traded_cross_rate=123,
            traded_swap_points=123,
            reference_spot_rate=123,
            ndf_fixing_settlement_ccy="",
            legs=[
                rdf.cross.LegDefinition(
                    deal_amount=1000000,
                    contra_amount=65762500,
                    deal_ccy_buy_sell=rdf.cross.BuySell.BUY,
                    tenor="4Y",
                )
            ],
            pricing_parameters=rdf.cross.PricingParameters(
                valuation_date="2017-11-15T00:00:00Z",
            ),
            fields=fields,
        ).get_data()

        check_http_status_is_success_and_df_value_not_empty(response)

    @allure.title(
        "Get async cross analytics Non-Deliverable Fx Forward Outright USD/INR"
    )
    @pytest.mark.caseid("36597407")
    async def test_get_async_cross_analytics_fx_non_deliverable_forward(
        self, open_session_async
    ):
        response = await get_async_response_from_definition(
            rdf.cross.Definition(
                instrument_tag="1Y-USDINR",
                fx_cross_type=rdf.cross.FxCrossType.FX_NON_DELIVERABLE_FORWARD,
                fx_cross_code="USDINR",
                traded_cross_rate=123,
                traded_swap_points=123,
                reference_spot_rate=123,
                ndf_fixing_settlement_ccy="",
                legs=[
                    rdf.cross.LegDefinition(
                        deal_amount=1000000,
                        contra_amount=65762500,
                        deal_ccy_buy_sell=rdf.cross.BuySell.BUY,
                        tenor="4Y",
                    )
                ],
                pricing_parameters=rdf.cross.PricingParameters(
                    valuation_date="2017-11-15T00:00:00Z",
                ),
                fields=fields,
            )
        )

        check_http_status_is_success_and_df_value_not_empty(response)

    @allure.title("Create a cross analytics FX Forward USD/EUR stream")
    @pytest.mark.caseid("36598198")
    def test_create_cross_analytics_fx_forward_stream(self, open_session):
        stream = rdf.cross.Definition(
            instrument_tag="00102700008910C",
            fx_cross_type=rdf.cross.FxCrossType.FX_FORWARD,
            fx_cross_code="USDEUR",
            traded_cross_rate=123,
            traded_swap_points=123,
            reference_spot_rate=123,
            ndf_fixing_settlement_ccy="",
            legs=[rdf.cross.LegDefinition(end_date="2015-04-09T00:00:00Z")],
            pricing_parameters=rdf.cross.PricingParameters(
                valuation_date="2015-02-02T00:00:00Z",
                price_side=rdf.cross.PriceSide.MID,
            ),
            fields=fields,
            extended_params={"instrumentDefinition": {"fxCrossCode": "CHFJPY"}},
        ).get_stream(session=open_session)

        stream.open()

        check_stream_state_and_df_from_stream(stream=stream, expected_value="CHFJPY")

    @allure.title(
        "Create invalid cross analytics Non-Deliverable Fx Forward Outright USD/INR and get RDError"
    )
    @pytest.mark.caseid("38515599")
    def test_create_invalid_cross_analytics_fx_non_deliverable_forward(
        self, open_session
    ):
        with pytest.raises(RDError) as error:
            rdf.cross.Definition(
                instrument_tag="1Y-USDINR",
                fields=fields,
            ).get_data()
        assert (
            str(error.value)
            == "Error code 400 | The fxCrossType cannot be found, in the request, from the expected path instrumentDefinition.fxCrossType for 'instrumentType' FxCross"
        )

    @allure.title("Create a cross definition with extended parameters")
    @pytest.mark.caseid("C38515601")
    def test_get_cross_with_extended_params(self, open_session):
        response = rdf.cross.Definition(
            fx_cross_type=rdf.cross.FxCrossType.FX_SWAP,
            fx_cross_code="USDINR",
            legs=[
                rdf.cross.LegDefinition(
                    contra_amount=1234,
                    deal_ccy_buy_sell=rdf.cross.BuySell.SELL,
                    tenor="10Y",
                )
            ],
            fields=fields,
            extended_params={
                "instrumentDefinition": {
                    "legs": [
                        {
                            "tenor": "4Y",
                            "dealCcyBuySell": "Buy",
                            "contraAmount": 65762500,
                            "dealAmount": 1000000,
                        }
                    ],
                    "fxCrossType": "FxNonDeliverableForward",
                    "fxCrossCode": "USDINR",
                },
                "pricingParameters": {"valuationDate": "2017-11-15T00:00:00Z"},
            },
        ).get_data()

        check_http_status_is_success_and_df_value_not_empty(response)

        request = json.loads(response.request_message.content.decode("utf-8"))
        request_universe = request.get("universe")[0]
        assert (
            request_universe == cross_universe
        ), f"Extended params are applied incorrectly"
